// Based on the original file <DemoApplication.cpp>
/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/
/*
This file has nothing to do with the demo itself, it's just a hack into BulletOpenGLSupport.lib
to display things in a better way (for the purpose of this demo).
*/
#include "GlutDemoApplicationEx.h"



// Copied and modified from GL_ShapeDrawer.cpp
// ... tons of code for just a few lines of modifications ...
#ifdef _WIN32 //needed for glut.h
#include <windows.h>
#endif
#include "GLDebugFont.h"
#include "GlutStuff.h"
#include "GL_ShapeDrawer.h"
#include "BulletCollision/CollisionShapes/btPolyhedralConvexShape.h"
#include "BulletCollision/CollisionShapes/btTriangleMeshShape.h"
#include "BulletCollision/CollisionShapes/btBoxShape.h"
#include "BulletCollision/CollisionShapes/btSphereShape.h"
#include "BulletCollision/CollisionShapes/btConeShape.h"
#include "BulletCollision/CollisionShapes/btCylinderShape.h"
#include "BulletCollision/CollisionShapes/btTetrahedronShape.h"
#include "BulletCollision/CollisionShapes/btCompoundShape.h"
#include "BulletCollision/CollisionShapes/btCapsuleShape.h"
#include "BulletCollision/CollisionShapes/btConvexTriangleMeshShape.h"
#include "BulletCollision/CollisionShapes/btUniformScalingShape.h"
#include "BulletCollision/CollisionShapes/btStaticPlaneShape.h"
#include "BulletCollision/CollisionShapes/btMultiSphereShape.h"
#include "BulletCollision/CollisionShapes/btConvexPolyhedron.h"
#include "BulletCollision/CollisionShapes/btConvexHullShape.h"
///
#include "BulletCollision/CollisionShapes/btShapeHull.h"

#include "LinearMath/btTransformUtil.h"


#include "LinearMath/btIDebugDraw.h"
#include "btTriangleMeshEx.h"
#include "LinearMath/btConvexHullComputer.h"

// ------- btUtils --------------------------
#include <LinearMath/btSerializer.h>
#include <btBulletDynamicsCommon.h>

bool GlutDemoApplicationEx::btUtils::Save(const btCollisionShape* shape,const char* filename)	{
 	if (!shape) return false;
 	
 	btDefaultSerializer serializer;
 	
 	//serializer.registerNameForPointer(shape, "BCS");
	
	serializer.startSerialization();
	
	shape->serializeSingleShape(&serializer);
	
	serializer.finishSerialization();  	

   	FILE* file = fopen(filename,"wb");
   	if (file)	{
		fwrite(serializer.getBufferPointer(),serializer.getCurrentBufferSize(),1, file);
   		fclose(file);
   	}
   	else return false;	
   	
 	return true;
}
btCompoundShape* GlutDemoApplicationEx::btUtils::GetUniformScalingCompoundShape(const btCompoundShape* cs,const btScalar uniformScalingFactor,const bool deepCloneNestedCompoundsWhenUniformScalingFactorIsOne)	{
	if (!cs) return NULL;
	
	btCompoundShape* newCs = new btCompoundShape();
	btTransform T;
	btCollisionShape* childShape;
	for (int i=0,sz=cs->getNumChildShapes();i<sz;i++)	{				
                childShape = const_cast < btCollisionShape* > (cs->getChildShape(i));	// const_cast because btCompoundShape::addChildShape(...) for some reasons takes a non-const btCollisionShape* as an argument
		T = cs->getChildTransform(i);
		T.getOrigin()*=uniformScalingFactor;
		if (childShape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE)	{
			newCs->addChildShape(T,(uniformScalingFactor!=1 || deepCloneNestedCompoundsWhenUniformScalingFactorIsOne) ? btUtils::GetUniformScalingCompoundShape(static_cast < btCompoundShape* > (childShape),uniformScalingFactor,deepCloneNestedCompoundsWhenUniformScalingFactorIsOne) : childShape);
			continue;
		}
		else if (btBroadphaseProxy::isConvex(childShape->getShapeType())) newCs->addChildShape(T,uniformScalingFactor!=1 ? new btUniformScalingShape(static_cast < btConvexShape* > (childShape),uniformScalingFactor) : childShape);
		else newCs->addChildShape(T,childShape);
	}
	return newCs;
}

btVector3 GlutDemoApplicationEx::btUtils::GetAabbFullExtents(const btCollisionShape* shape)	{
	if (shape)	{
		btVector3 aabbMin,aabbMax;
		shape->getAabb(btTransform::getIdentity(),aabbMin,aabbMax);
		return (aabbMax - aabbMin);		
	}	
	return btVector3(0,0,0);
}
btVector3 GlutDemoApplicationEx::btUtils::GetAabbHalfExtents(const btCollisionShape* shape)	{
	return GetAabbFullExtents(shape)*btScalar(0.5);
}
//-------------------------------------------
class Matrix16Helper	{
public:

template < typename T > SIMD_FORCE_INLINE static void Multiply(const T* b,const T* a,T* res) {
        res[ 0] = a[ 0] * b[ 0] + a[ 1] * b[ 4] + a[ 2] * b[ 8] + a[ 3] * b[12];
        res[ 1] = a[ 0] * b[ 1] + a[ 1] * b[ 5] + a[ 2] * b[ 9] + a[ 3] * b[13];
        res[ 2] = a[ 0] * b[ 2] + a[ 1] * b[ 6] + a[ 2] * b[10] + a[ 3] * b[14];
        res[ 3] = a[ 0] * b[ 3] + a[ 1] * b[ 7] + a[ 2] * b[11] + a[ 3] * b[15];

        res[ 4] = a[ 4] * b[ 0] + a[ 5] * b[ 4] + a[ 6] * b[ 8] + a[ 7] * b[12];
        res[ 5] = a[ 4] * b[ 1] + a[ 5] * b[ 5] + a[ 6] * b[ 9] + a[ 7] * b[13];
        res[ 6] = a[ 4] * b[ 2] + a[ 5] * b[ 6] + a[ 6] * b[10] + a[ 7] * b[14];
        res[ 7] = a[ 4] * b[ 3] + a[ 5] * b[ 7] + a[ 6] * b[11] + a[ 7] * b[15];

        res[ 8] = a[ 8] * b[ 0] + a[ 9] * b[ 4] + a[10] * b[ 8] + a[11] * b[12];
        res[ 9] = a[ 8] * b[ 1] + a[ 9] * b[ 5] + a[10] * b[ 9] + a[11] * b[13];
        res[10] = a[ 8] * b[ 2] + a[ 9] * b[ 6] + a[10] * b[10] + a[11] * b[14];
        res[11] = a[ 8] * b[ 3] + a[ 9] * b[ 7] + a[10] * b[11] + a[11] * b[15];

        res[12] = a[12] * b[ 0] + a[13] * b[ 4] + a[14] * b[ 8] + a[15] * b[12];
        res[13] = a[12] * b[ 1] + a[13] * b[ 5] + a[14] * b[ 9] + a[15] * b[13];
        res[14] = a[12] * b[ 2] + a[13] * b[ 6] + a[14] * b[10] + a[15] * b[14];
        res[15] = a[12] * b[ 3] + a[13] * b[ 7] + a[14] * b[11] + a[15] * b[15];
        }
template < typename T > SIMD_FORCE_INLINE static void GetGlFrustumMatrix(T left,T right,T bottom,T top,T znear,T zfar,T* m)	{
  				static const T eps = SIMD_EPSILON;
 				T idx = (right-left);
 				T idy = (top-bottom);
 				T idz = (zfar-znear);
 				idx = idx == 0 ? 1.0/eps : 1.0/idx;
 				idy = idy == 0 ? 1.0/eps : 1.0/idy;
 				idz = idz == 0 ? 1.0/eps : 1.0/idz;
 				
 				const T A = (right+left)*idx;
				const T B = (top+bottom)*idy;
				const T C = - (zfar+znear)*idz;		
				const T D = - 2.0*zfar*znear*idz;		
		
				const T e00 = 2.0*znear*idx;
				const T e11 = 2.0*znear*idy;
				
				/*
				e00		0		A		0
				0		e11		B		0
				0		0		C		D
				0		0		-1		0
				*/	
				m[0] = e00;
				m[1] = m[2] = m[3] = 0;
				
				m[4] = m[6] = m[7] = 0;
				m[5] = e11;
				
				m[8] = A;
				m[9] = B;
				m[10] = C;
				m[11] = -1;
				
				m[12] = m[13] = m[15] = 0;
				m[14] = D;				  			
  		}
template < typename T > SIMD_FORCE_INLINE static void GetGlOrthoMatrix(T left,T right,T bottom,T top,T znear,T zfar,T* m)	{
  				static const T eps = SIMD_EPSILON;
 				T idx = (right-left);
 				T idy = (top-bottom);
 				T idz = (zfar-znear);
 				idx = idx == 0 ? 1.0/eps : 1.0/idx;
 				idy = idy == 0 ? 1.0/eps : 1.0/idy;
 				idz = idz == 0 ? 1.0/eps : 1.0/idz;
 				
 				const T tx = - (right+left)*idx;
				const T ty = - (top+bottom)*idy;
				const T tz = - (zfar+znear)*idz;
		
				const T e00 = 2.0*idx;
				const T e11 = 2.0*idy;
				const T e22 = -2.0*idz; 
				
				/*
				e00		0		0		tx
				0		e11		0		ty
				0		0		e22		tz
				0		0		0		1
				*/
				m[0] = e00;
				m[1] = m[2] = m[3] =0;
				
				m[4] = m[6] = m[7] = 0;
				m[5] = e11;
				
				m[8] = m[9] = m[11] = 0;
				m[10] = e22;
				
				m[12] = tx;
				m[13] = ty;
				m[14] = tz;
				m[15] =   1;		
	}	
template < typename T > SIMD_FORCE_INLINE static void GetGluPerspectiveMatrix(T degFov,T aspect,T znear,T zfar,T* m)	{
		T w,h;
		const T FOVTG = tan(degFov*1.5707963268f/180.0);				
		h = znear * FOVTG;			
		w = h * aspect;
		
		GetGlFrustumMatrix(-w,w,-h,h,znear,zfar,m);		
	}
template < typename T > SIMD_FORCE_INLINE static void GetGluOrtho2DMatrix(T left,T right,T bottom,T top,T* m)	{
		GetGlOrthoMatrix(left,right,bottom,top,1,1,m);
	}
template < typename T > SIMD_FORCE_INLINE static void GetGluOrtho3DMatrix(T cameraTargetDistance,T degFOV,T aspect,T znear,T zfar,T* m)	{
		const T FOVTG=tan(degFOV*1.5707963268f/180.0);	
		T y;//=(zfar-znear)*0.5f;
		y=cameraTargetDistance*FOVTG;
		T x=y*aspect;
		GetGlOrthoMatrix( -x, x, -y, y, znear, zfar, m);
	}

protected:
Matrix16Helper() {}
};

int GlutDemoApplicationEx::g_colorMode(0);	
int GlutDemoApplicationEx::g_renderMode(0);	
bool GlutDemoApplicationEx::g_cullBackFaces(true);
btVector3 GlutDemoApplicationEx::g_shadowColor(0.2,0.2,0.2);
GlutDemoApplicationEx* GlutDemoApplicationEx::g_myGlutDemoApplicationEx(NULL);


GlutDemoApplicationEx::GlutDemoApplicationEx(const AlternativeNavigationSystemParams& alternativeNavigationSystemParams)
	: m_roll(0.0),m_alternativeNavigationSystemParams(alternativeNavigationSystemParams),
	  GlutDemoApplication()
{
		g_myGlutDemoApplicationEx = this;
		//---------------------------------------
		m_cameraT = m_alternativeNavigationSystemParams.getStartCameraTransform();
        // frustum culling stuff------------------
        m_frustumCullingEnabled = true;
        m_DbvtCullingCollisionFilterMask = btBroadphaseProxy::AllFilter & ~btBroadphaseProxy::SensorTrigger;
        //----------------------------------------
        m_fovDeg = btScalar(60.0);
        m_raytraceTextureId = 0;
        m_showRaytraceImage = false;  
        m_raytraceTextureMaxDimension = m_raytraceTextureWidth = m_raytraceTextureHeight = 256;
		m_raytraceTextureBuffer = NULL;
		//----------------------------------------		
		if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem) 	{
			m_azi = m_alternativeNavigationSystemParams.m_cameraStartYaw;
			m_ele = m_alternativeNavigationSystemParams.m_cameraStartPitch;
			m_cameraT.setRotation(btQuaternion(btRadians(m_azi),btRadians(m_ele),0));
		}		
		//----------------------------------------
		m_collisionConfiguration = NULL;
		m_dispatcher = NULL;
		m_broadphase = NULL;
		m_solver = NULL;
		m_dynamicsWorld = NULL;
		m_ghostPairCallback = NULL;	
		m_softBodyWorldInfo = NULL;	
		//----------------------------------------
		m_mouseButtons = -1;
		//----------------------------------------
		reset();
}	
	
GlutDemoApplicationEx::~GlutDemoApplicationEx()	{
		btTriangleMeshExFootPrint::Helper::ClearArray();
		DisplayListHelper::ClearArray();
		TextureHelper::ClearArray();	
		if (m_raytraceTextureId>0) {
			glDeleteTextures(1,&m_raytraceTextureId);
			m_raytraceTextureId = 0;
		}
		if (m_raytraceTextureBuffer) {
			delete[] m_raytraceTextureBuffer;
			m_raytraceTextureBuffer = NULL;
		}
}

void GlutDemoApplicationEx::reset()	{
	overrideGLShapeDrawer(new GL_MyShapeDrawer());	// Deleted by DemoApplication::dct()	// This line should call the previous GL_MyShapeDrawer::dctr() that clears all memory
	static_cast <GL_MyShapeDrawer* > (m_shapeDrawer)->m_areDefaultTextureSettingsEnabled=&this->m_areDefaultTextureSettingsEnabled;
	// OpenGL Stuff:
		
	setTexturing(true);
	setShadows(true);
        //m_inited = false;
    if (m_raytraceTextureId>0) {
		glDeleteTextures(1,&m_raytraceTextureId);
		m_raytraceTextureId = 0;
	}
	m_showRaytraceImage = false;    
	if (m_raytraceTextureBuffer) {
		delete[] m_raytraceTextureBuffer;
		m_raytraceTextureBuffer = NULL;
	} 
	//if (m_shootBoxShape)	{delete m_shootBoxShape;m_shootBoxShape = NULL;}
}	

static inline short GetDefaultCollisionFilterGroup(short _collisionFilterGroup,bool isDynamic)	{
	return  _collisionFilterGroup>=0 ? (isDynamic ? _collisionFilterGroup : (_collisionFilterGroup|btBroadphaseProxy::StaticFilter)) : (isDynamic? short(btBroadphaseProxy::DefaultFilter) : short(btBroadphaseProxy::StaticFilter));
}
static inline short GetDefaultCollisionFilterMask(short _collisionFilterMask,bool isDynamic)	{
	return  _collisionFilterMask>=0 ? (isDynamic ? _collisionFilterMask : (_collisionFilterMask&(~btBroadphaseProxy::StaticFilter))) : (isDynamic? short(btBroadphaseProxy::AllFilter) : short(btBroadphaseProxy::AllFilter ^ btBroadphaseProxy::StaticFilter));
}
btRigidBody*	GlutDemoApplicationEx::localCreateRigidBody(float mass, const btTransform& startTransform,btCollisionShape* shape,short int _collisionFilterGroup,short int _collisionFilterMask)
{
	btAssert((!shape || shape->getShapeType() != INVALID_SHAPE_PROXYTYPE));
	bool isDynamic = (mass != 0.f);
	btVector3 localInertia(0,0,0);
	if (isDynamic) shape->calculateLocalInertia(mass,localInertia);

	btDefaultMotionState* myMotionState = new btDefaultMotionState(startTransform);
	btRigidBody::btRigidBodyConstructionInfo cInfo(mass,myMotionState,shape,localInertia);

	btRigidBody* body = new btRigidBody(cInfo);
	body->setContactProcessingThreshold(m_defaultContactProcessingThreshold);

	const short collisionFilterGroup = GetDefaultCollisionFilterGroup(_collisionFilterGroup,isDynamic);
	const short collisionFilterMask = GetDefaultCollisionFilterMask(_collisionFilterMask,isDynamic);
	m_dynamicsWorld->addRigidBody(body,collisionFilterGroup,collisionFilterMask);
	return body;
}
btRigidBody*	GlutDemoApplicationEx::localCreateRigidBodyWithCOMOffset(float mass, const btTransform& startTransform,btCollisionShape* shapeWithNoCOMOffset,const btVector3& COMoffset,short int _collisionFilterGroup,short int _collisionFilterMask)
{
	btAssert((!shapeWithNoCOMOffset || shapeWithNoCOMOffset->getShapeType() != INVALID_SHAPE_PROXYTYPE));

	btCollisionShape* shape = shapeWithNoCOMOffset;
	const btTransform COMOffsetTransform(btTransform(btQuaternion::getIdentity(),COMoffset));	
	const btTransform inverseCOMOffsetTransform = COMOffsetTransform.inverse();//(btTransform(btQuaternion::getIdentity(),-COMoffset));
	const bool useCOMoffsetIndeed = COMoffset!=btVector3(0,0,0);
	if (useCOMoffsetIndeed)	{
		//Wrap it inside a btCompoundShape (TODO WARNING for unsupported shapes)
		btCompoundShape* cs = new btCompoundShape();
		cs->addChildShape(inverseCOMOffsetTransform,shapeWithNoCOMOffset);
		shape = cs;
	}

	bool isDynamic = (mass != 0.f);
	btVector3 localInertia(0,0,0);
	if (isDynamic)	shape->calculateLocalInertia(mass,localInertia);

	btDefaultMotionState* myMotionState;
	if (!useCOMoffsetIndeed)	myMotionState = new btDefaultMotionState(startTransform);
	else	{
		#ifndef USE_COLLISION_OBJECT_USER_POINTER
		const btTransform transformedStartTransform(startTransform.getBasis(),startTransform.getOrigin()+startTransform.getBasis()*(COMoffset));		
		myMotionState = new btDefaultMotionState(transformedStartTransform);
		#else //USE_COLLISION_OBJECT_USER_POINTER
		myMotionState = new btMyDefaultMotionState(startTransform,COMoffset);
		#endif//USE_COLLISION_OBJECT_USER_POINTER
	}	
	
	btRigidBody::btRigidBodyConstructionInfo cInfo(mass,myMotionState,shape,localInertia);	

	btRigidBody* body = new btRigidBody(cInfo);
	if (body)	{
		body->setContactProcessingThreshold(m_defaultContactProcessingThreshold);
		/*
		if (useCOMoffsetIndeed)	{
			#ifdef USE_COLLISION_OBJECT_USER_POINTER
			btTriangleMeshExFootPrint::Overrides& overrides = GlutDemoApplicationEx::GetMeshOverrides(*body);
			overrides.offset = COMoffset;
			#endif //USE_COLLISION_OBJECT_USER_POINTER
		}
		*/
		const short collisionFilterGroup = GetDefaultCollisionFilterGroup(_collisionFilterGroup,isDynamic);
		const short collisionFilterMask = GetDefaultCollisionFilterMask(_collisionFilterMask,isDynamic);
		m_dynamicsWorld->addRigidBody(body,collisionFilterGroup,collisionFilterMask);
		
		/*
		btMyDefaultMotionState* ms = btMyDefaultMotionState::upcast(body);
		if (ms) printf("Using btMyDefaultMotionState\n");
		else printf("Using btDefaultMotionState\n");
		*/
	}
	
	return body;
}

class GlDrawcallback : public btTriangleCallback
{

public:

	bool	m_wireframe;

	GlDrawcallback()
		:m_wireframe(false)
	{
	}

	virtual void processTriangle(btVector3* triangle,int partId, int triangleIndex)
	{

		(void)triangleIndex;
		(void)partId;


		if (m_wireframe)
		{
			glBegin(GL_LINES);
			glColor3f(1, 0, 0);
			glVertex3d(triangle[0].getX(), triangle[0].getY(), triangle[0].getZ());
			glVertex3d(triangle[1].getX(), triangle[1].getY(), triangle[1].getZ());
			glColor3f(0, 1, 0);
			glVertex3d(triangle[2].getX(), triangle[2].getY(), triangle[2].getZ());
			glVertex3d(triangle[1].getX(), triangle[1].getY(), triangle[1].getZ());
			glColor3f(0, 0, 1);
			glVertex3d(triangle[2].getX(), triangle[2].getY(), triangle[2].getZ());
			glVertex3d(triangle[0].getX(), triangle[0].getY(), triangle[0].getZ());
			glEnd();
		} else
		{
			glBegin(GL_TRIANGLES);
			//glColor3f(1, 1, 1);
			
			
			glVertex3d(triangle[0].getX(), triangle[0].getY(), triangle[0].getZ());
			glVertex3d(triangle[1].getX(), triangle[1].getY(), triangle[1].getZ());
			glVertex3d(triangle[2].getX(), triangle[2].getY(), triangle[2].getZ());

			glVertex3d(triangle[2].getX(), triangle[2].getY(), triangle[2].getZ());
			glVertex3d(triangle[1].getX(), triangle[1].getY(), triangle[1].getZ());
			glVertex3d(triangle[0].getX(), triangle[0].getY(), triangle[0].getZ());
			glEnd();
		}
	}
};

inline static void glDrawVector(const btVector3& v)
{ glVertex3d(v[0], v[1], v[2]);
}


GL_MyShapeDrawer::MyShapeCache::MyShapeCache(btConvexShape* s) : shapeCache(s), customMesh(NULL),customMeshGuest(NULL),overridesGuest(NULL),userPointer(NULL) {
	dl[0]=dl[1]=0;
}
GL_MyShapeDrawer::MyShapeCache::~MyShapeCache()
{
	//if (customMesh) btTriangleMeshEx::DeleteMeshFootPrint(customMesh);
}



GL_MyShapeDrawer::MyShapeCache*		GL_MyShapeDrawer::mycache(btCollisionShape* shape)
{
	MyShapeCache*		mysc=(MyShapeCache*)shape->getUserPointer();
	if(!mysc)
	{
		//printf("Start creating myshapecache for: %s\n",shape->getName());
		mysc=new(btAlignedAlloc(sizeof(MyShapeCache),16)) MyShapeCache((btConvexShape*)shape);
                m_myshapecaches.push_back(mysc);
		ShapeCache* sc = &mysc->shapeCache;
                shape->setUserPointer(mysc);
		if (shape->isConvex())	{
		sc->m_shapehull.buildHull(shape->getMargin());
		/* Build edges	*/ 
		const int			ni=sc->m_shapehull.numIndices();
		const int			nv=sc->m_shapehull.numVertices();
		const unsigned int*	pi=sc->m_shapehull.getIndexPointer();
		const btVector3*	pv=sc->m_shapehull.getVertexPointer();
		btAlignedObjectArray<ShapeCache::Edge*>	edges;
		sc->m_edges.reserve(ni);
		edges.resize(nv*nv,0);
		for(int i=0;i<ni;i+=3)
		{
			const unsigned int* ti=pi+i;
			const btVector3		nrm=btCross(pv[ti[1]]-pv[ti[0]],pv[ti[2]]-pv[ti[0]]).normalized();
			for(int j=2,k=0;k<3;j=k++)
			{
				const unsigned int	a=ti[j];
				const unsigned int	b=ti[k];
				ShapeCache::Edge*&	e=edges[btMin(a,b)*nv+btMax(a,b)];
				if(!e)
				{
					sc->m_edges.push_back(ShapeCache::Edge());
					e=&sc->m_edges[sc->m_edges.size()-1];
					e->n[0]=nrm;e->n[1]=-nrm;
					e->v[0]=a;e->v[1]=b;
				}
				else
				{
					e->n[1]=nrm;
				}
			}
		}
		
		// Build dl[0] = btShapeHull (Bullet default)
		mysc->dl[0] = glGenLists(1);
		glNewList(mysc->dl[0],GL_COMPILE);
		{
							btShapeHull* hull = &sc->m_shapehull/*(btShapeHull*)shape->getUserPointer()*/;

							if (hull->numTriangles () > 0)
							{
								int index = 0;
								const unsigned int* idx = hull->getIndexPointer();
								const btVector3* vtx = hull->getVertexPointer();

								glBegin (GL_TRIANGLES);

								for (int i = 0; i < hull->numTriangles (); i++)
								{
									int i1 = index++;
									int i2 = index++;
									int i3 = index++;
									btAssert(i1 < hull->numIndices () &&
										i2 < hull->numIndices () &&
										i3 < hull->numIndices ());

									int index1 = idx[i1];
									int index2 = idx[i2];
									int index3 = idx[i3];
									btAssert(index1 < hull->numVertices () &&
										index2 < hull->numVertices () &&
										index3 < hull->numVertices ());

									btVector3 v1 = vtx[index1];
									btVector3 v2 = vtx[index2];
									btVector3 v3 = vtx[index3];
									btVector3 normal = (v3-v1).cross(v2-v1);
									normal.normalize ();
									glNormal3f(normal.getX(),normal.getY(),normal.getZ());
									glVertex3f (v1.x(), v1.y(), v1.z());
									glVertex3f (v2.x(), v2.y(), v2.z());
									glVertex3f (v3.x(), v3.y(), v3.z());

								}
								glEnd ();

							}		
		}
		glEndList();
		
		DisplayListHelper::Add(mysc->dl[0]);
		
		// Build dl[1]: = btConvexHullComputer (btConvexHullShape only) 
		if (shape->isPolyhedral() && static_cast < const btPolyhedralConvexShape* > (shape)->getConvexPolyhedron())	{
			// We can speed up things in this case (but this if block can be removed)
			const btPolyhedralConvexShape* pcshape(static_cast < const btPolyhedralConvexShape* > (shape));
			const btConvexPolyhedron& cp( *(pcshape->getConvexPolyhedron()) );
			
			// Build dl[1]:
			mysc->dl[1] = glGenLists(1);
			glNewList(mysc->dl[1],GL_COMPILE);	
			{
				// TRIANGLES, QUADS and POLYGONS (but we could triangulate all the faces...)
				{
				bool anyTriangle(false);	// we incorporate quads too...
				for (int i=0,sz = cp.m_faces.size();i<sz;i++)	{
					const btFace& f(cp.m_faces[i]);
					if (f.m_indices.size()==3 || f.m_indices.size()==4)	{
						if (!anyTriangle)	{
							anyTriangle = true;
							glBegin(GL_TRIANGLES);
						}
						{
						const btVector3& V1 = cp.m_vertices[f.m_indices[0]];
						const btVector3& V2 = cp.m_vertices[f.m_indices[1]];
						const btVector3& V3 = cp.m_vertices[f.m_indices[2]];
						
						btVector3 normal = (V3-V1).cross(V2-V1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
						glVertex3f (V3.x(), V3.y(), V3.z());											
						}
						if (f.m_indices.size()==4)	{
						const btVector3& V1 = cp.m_vertices[f.m_indices[2]];
						const btVector3& V2 = cp.m_vertices[f.m_indices[3]];
						const btVector3& V3 = cp.m_vertices[f.m_indices[0]];
						
						btVector3 normal = (V3-V1).cross(V2-V1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
						glVertex3f (V3.x(), V3.y(), V3.z());											
						}						
					}
				}
				if (anyTriangle) glEnd();
				}
				// POLYGONS
				{
				for (int i=0,sz = cp.m_faces.size();i<sz;i++)	{
					const btFace& f(cp.m_faces[i]);
					if (f.m_indices.size()>4)	{
						glBegin(GL_POLYGON);
						{
						const btVector3& V1 = cp.m_vertices[f.m_indices[0]];
						const btVector3& V2 = cp.m_vertices[f.m_indices[1]];
						const btVector3& V3 = cp.m_vertices[f.m_indices[2]];
						
						btVector3 normal = (V3-V1).cross(V2-V1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
						glVertex3f (V3.x(), V3.y(), V3.z());
						
						for (int j=3;j<f.m_indices.size();j++)	{
							const btVector3& V = cp.m_vertices[f.m_indices[j]];
							glVertex3f (V.x(), V.y(), V.z());
						}																	
						}	
						glEnd();					
					}
				}
				}
			}		
			glEndList();		
			DisplayListHelper::Add(mysc->dl[1]);
		}
		else if (shape->getShapeType()==CONVEX_HULL_SHAPE_PROXYTYPE)	{
		//-------------------------------------------
		btConvexHullComputer	computer;// = mysc->convexHullComputer;
		const btConvexHullShape* chshape(static_cast < const btConvexHullShape* > (shape));
		const btVector3* vertices = chshape->getUnscaledPoints();
		int stride = sizeof(btVector3);
		int count = chshape->getNumPoints();
		btScalar shrink= 0.0;//chshape->getMargin();	// 0.0 ?
		btScalar shrinkClamp=0.f;
		if (computer.compute(&vertices[0].getX(),stride,count,shrink,shrinkClamp)<=0 && shrink>0)	computer.compute(&vertices[0].getX(),stride,count,0.,0.);
		
		// Build dl[1]:
		mysc->dl[1] = glGenLists(1);
		glNewList(mysc->dl[1],GL_COMPILE);
		{	
			//#define DRAW_POLYGONS		// simpler and probably slower
			#ifndef DRAW_POLYGONS		// same quality, but should be faster, far less glBegin()/glEnd() calls (to be honest it should be possible to triangulate all faces too).
			{
			// DRAW TRIANGLES
			{
				bool start = false;
				for (int i=0;i<computer.faces.size();i++)	{
					int face = computer.faces[i];
					const btConvexHullComputer::Edge*  firstEdge = &computer.edges[face];
					const btConvexHullComputer::Edge*  edge = firstEdge;
					int cnt = 2;
					while (edge->getNextEdgeOfFace()!=firstEdge) {
						++cnt;edge = edge->getNextEdgeOfFace();						
					}
					if (cnt == 3)	{
						if (!start)	{
							start = true;
							glBegin(GL_TRIANGLES);
						}
						edge = firstEdge;
						int v0 = edge->getSourceVertex();
						int v1 = edge->getTargetVertex();
						int v2 = edge->getNextEdgeOfFace()->getTargetVertex();
						
						const btVector3& V1 = computer.vertices[v0];
						const btVector3& V2 = computer.vertices[v1];
						const btVector3& V3 = computer.vertices[v2];
						
						btVector3 normal = (V3-V1).cross(V2-V1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
						glVertex3f (V3.x(), V3.y(), V3.z());						
					}
				}
				if (start) glEnd();
			}
			// DRAW QUADS
			{
				bool start = false;
				for (int i=0;i<computer.faces.size();i++)	{
					int face = computer.faces[i];
					const btConvexHullComputer::Edge*  firstEdge = &computer.edges[face];
					const btConvexHullComputer::Edge*  edge = firstEdge;
					int cnt = 2;
					while (edge->getNextEdgeOfFace()!=firstEdge) {
						++cnt;edge = edge->getNextEdgeOfFace();						
					}
					if (cnt == 4)	{
						if (!start)	{
							start = true;
							glBegin(GL_QUADS);
						}
						edge = firstEdge;
						int v0 = edge->getSourceVertex();
						int v1 = edge->getTargetVertex();
						edge = edge->getNextEdgeOfFace();
						int v2 = edge->getTargetVertex();
						edge = edge->getNextEdgeOfFace();
						int v3 = edge->getTargetVertex();
						const btVector3& V1 = computer.vertices[v0];
						const btVector3& V2 = computer.vertices[v1];
						const btVector3& V3 = computer.vertices[v2];
						const btVector3& V4 = computer.vertices[v3];
						
						btVector3 normal = (V3-V1).cross(V2-V1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
						glVertex3f (V3.x(), V3.y(), V3.z());
						glVertex3f (V4.x(), V4.y(), V4.z());						
					}
				}
				if (start) glEnd();			
			}
			// DRAW POLYGONS
			{
				for (int i=0;i<computer.faces.size();i++)	{
					int face = computer.faces[i];
					const btConvexHullComputer::Edge*  firstEdge = &computer.edges[face];
					const btConvexHullComputer::Edge*  edge = firstEdge;
					int cnt = 2;
					while (edge->getNextEdgeOfFace()!=firstEdge) {
						++cnt;edge = edge->getNextEdgeOfFace();						
					}
					if (cnt > 4)	{
						glBegin(GL_POLYGON);
						int face = computer.faces[i];
						const btConvexHullComputer::Edge*  firstEdge = &computer.edges[face];
						const btConvexHullComputer::Edge*  edge = firstEdge;
					
						int v0 = edge->getSourceVertex();
						int v1 = edge->getTargetVertex();
						int v2 = edge->getNextEdgeOfFace()->getTargetVertex();
						{
							const btVector3& V1 = computer.vertices[v0];
							const btVector3& V2 = computer.vertices[v1];
							const btVector3& V3 = computer.vertices[v2];
						
							btVector3 normal = (V3-V1).cross(V2-V1);
							normal.normalize ();
							glNormal3f(normal.getX(),normal.getY(),normal.getZ());
							glVertex3f (V1.x(), V1.y(), V1.z());
							glVertex3f (V2.x(), V2.y(), V2.z());
						}
						while (edge->getNextEdgeOfFace()!=firstEdge)
						{
							edge = edge->getNextEdgeOfFace();
							v0 = edge->getSourceVertex();
							v1 = edge->getTargetVertex();
							const btVector3& V1 = computer.vertices[v0];
							const btVector3& V2 = computer.vertices[v1];
						
							glVertex3f (V1.x(), V1.y(), V1.z());
							glVertex3f (V2.x(), V2.y(), V2.z());
						};
						glEnd();						
					}
				}			
			}
			}
			#else //DRAW_POLYGONS
			for (int i=0;i<computer.faces.size();i++)
				{
					glBegin(GL_POLYGON);
					int face = computer.faces[i];
					const btConvexHullComputer::Edge*  firstEdge = &computer.edges[face];
					const btConvexHullComputer::Edge*  edge = firstEdge;
					
					int v0 = edge->getSourceVertex();
					int v1 = edge->getTargetVertex();
					int v2 = edge->getNextEdgeOfFace()->getTargetVertex();
					{
						const btVector3& V1 = computer.vertices[v0];
						const btVector3& V2 = computer.vertices[v1];
						const btVector3& V3 = computer.vertices[v2];
						
						btVector3 normal = (V3-V1).cross(V2-V1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
					}
					while (edge->getNextEdgeOfFace()!=firstEdge)
					{
						edge = edge->getNextEdgeOfFace();
						v0 = edge->getSourceVertex();
						v1 = edge->getTargetVertex();
						const btVector3& V1 = computer.vertices[v0];
						const btVector3& V2 = computer.vertices[v1];
						
						glVertex3f (V1.x(), V1.y(), V1.z());
						glVertex3f (V2.x(), V2.y(), V2.z());
					};
					glEnd();
				}			
			#undef DRAW_POLYGONS
			#endif //DRAW_POLYGONS
						
		}
		glEndList();		
		DisplayListHelper::Add(mysc->dl[1]);
		//-------------------------------------------
        }

		//printf("End creating myshapecache\n");
		}	
	}
	return(mysc);
}

GL_MyShapeDrawer::~GL_MyShapeDrawer()
{
		//printf("Start: GL_MyShapeDrawer::~GL_MyShapeDrawer()\n");
		int i;
		for (i=0;i<m_myshapecaches.size();i++)
		{
			MyShapeCache* msc = m_myshapecaches[i];
			if (msc)	{
				if (msc->userPointer && deleteCollisionShapeUserPointerCallback) (*deleteCollisionShapeUserPointerCallback)(msc->userPointer);
				msc->~MyShapeCache();
				btAlignedFree(msc);
                                msc=NULL;
			}	
		}
		m_myshapecaches.clear();
		btTriangleMeshExFootPrint::Helper::ClearArray();
		DisplayListHelper::ClearArray();
		TextureHelper::ClearArray();
		//printf("End: GL_MyShapeDrawer::~GL_MyShapeDrawer()\n");
		
}


void GlutDemoApplicationEx::AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling, const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArray,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray)	{
	if (!shape) return;
	//printf("void GlutDemoApplicationEx::AssignMesh(...)\n");
	GL_MyShapeDrawer::MyShapeCache*	mysc = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(shape);

	mesh.createMeshFootPrint(mysc->customMesh,scaling,useFlatNormals,pOptionalMaterialOverrideArray,pOptionalTextureOverrideArray,pOptionalSubmeshIndexArray);
	//printf("End void GlutDemoApplicationEx::AssignMesh(...)\n");
}
void GlutDemoApplicationEx::AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride)	{
	btAlignedObjectArray< const Material* > optionalMaterialOverrideArray;
	optionalMaterialOverrideArray.resize(1);
	optionalMaterialOverrideArray[0] = &firstMaterialOverride;	// it should be OK to pass ptrs happily this way: the material is deep cloned.
        AssignMesh(shape,mesh,useFlatNormals,scaling,&optionalMaterialOverrideArray);
}
void GlutDemoApplicationEx::AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const GLuint firstTextureIdOverride)	{
	btAlignedObjectArray< GLuint > optionalTextureOverrideArray;
	optionalTextureOverrideArray.resize(1);
	optionalTextureOverrideArray[0] = firstTextureIdOverride;	
        AssignMesh(shape,mesh,useFlatNormals,scaling,NULL,&optionalTextureOverrideArray);
}
void GlutDemoApplicationEx::AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride,const GLuint firstTextureIdOverride)	{
	btAlignedObjectArray< const Material* > optionalMaterialOverrideArray;
	optionalMaterialOverrideArray.resize(1);
	optionalMaterialOverrideArray[0] = &firstMaterialOverride;	// it should be OK to pass ptrs happily this way: the material is deep cloned.
	btAlignedObjectArray< GLuint > optionalTextureOverrideArray;
	optionalTextureOverrideArray.resize(1);
	optionalTextureOverrideArray[0] = firstTextureIdOverride;
        AssignMesh(shape,mesh,useFlatNormals,scaling,&optionalMaterialOverrideArray,&optionalTextureOverrideArray);
}
void GlutDemoApplicationEx::AssignMeshCopy(btCollisionShape* shapeTo,const btCollisionShape* shapeFrom)	{
	if (!shapeTo || !shapeFrom) return;
	const GL_MyShapeDrawer::MyShapeCache*	myscFrom = const_cast < const GL_MyShapeDrawer::MyShapeCache* > (static_cast < GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(const_cast < btCollisionShape*> (shapeFrom)));
	GL_MyShapeDrawer::MyShapeCache*	myscTo = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(shapeTo);
	if (!myscFrom || !myscTo) return;
	myscTo->customMesh = myscFrom->customMesh;
	myscTo->overrides = myscFrom->overrides;	
}
void GlutDemoApplicationEx::SetUserPointer(btCollisionShape* shape,void* userPointer)	{
	if (!shape) return;
	GL_MyShapeDrawer::MyShapeCache*	mysc = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(shape);
	if (mysc) mysc->userPointer = userPointer;
}
void* GlutDemoApplicationEx::GetUserPointer(const btCollisionShape* shape) const	{
	if (!shape || !shape->getUserPointer()) return NULL;
	GL_MyShapeDrawer::MyShapeCache*	mysc = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(const_cast <btCollisionShape*> (shape));
	if (!mysc) return NULL;
	return mysc->userPointer;
}
void GlutDemoApplicationEx::SetDeleteCollisionShapeUserPointerCallback(GlutDemoApplicationEx::DeleteCollisionShapeUserPointerCallback callback)	{
	if (m_shapeDrawer)	{
		static_cast <GL_MyShapeDrawer*> (m_shapeDrawer)->setDeleteCollisionShapeUserPointerCallback(callback);
	}	
}
const btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(const btCollisionShape& myShape) const	{
	btCollisionShape* shape = const_cast < btCollisionShape* > (&myShape);
	GL_MyShapeDrawer::MyShapeCache*	mysc = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(shape);
	return mysc->overrides;
}
btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(btCollisionShape& shape)	{
	GL_MyShapeDrawer::MyShapeCache*	mysc = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(&shape);
	return mysc->overrides;	
}

void GlutDemoApplicationEx::myinit(void)
{
        GlutDemoApplication::myinit();
        /*
        //--------------------------
        // DemoApplication::myinit():
        //----------------------------
        GLfloat light_ambient[] = { btScalar(0.2), btScalar(0.2), btScalar(0.2), btScalar(1.0) };
        GLfloat light_diffuse[] = { btScalar(1.0), btScalar(1.0), btScalar(1.0), btScalar(1.0) };
        GLfloat light_specular[] = { btScalar(1.0), btScalar(1.0), btScalar(1.0), btScalar(1.0 )};
        //	light_position is NOT default value
        GLfloat light_position0[] = { btScalar(1.0), btScalar(10.0), btScalar(1.0), btScalar(0.0 )};
        GLfloat light_position1[] = { btScalar(-1.0), btScalar(-10.0), btScalar(-1.0), btScalar(0.0) };

        glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
        glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
        glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
        glLightfv(GL_LIGHT0, GL_POSITION, light_position0);

        glLightfv(GL_LIGHT1, GL_AMBIENT, light_ambient);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, light_diffuse);
        glLightfv(GL_LIGHT1, GL_SPECULAR, light_specular);
        glLightfv(GL_LIGHT1, GL_POSITION, light_position1);

        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_LIGHT1);    // 2 lights are active:maybe we can switch this one off for a small speed up.


        glShadeModel(GL_SMOOTH);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LESS);

        glEnable(GL_CULL_FACE);
        glCullFace(GL_BACK);
        */
        glClearColor(btScalar(0.6),btScalar(0.8),btScalar(1.0),btScalar(1));
        glEnable(GL_TEXTURE_2D);
        glDisable(GL_LIGHT1);
}

void GlutDemoApplicationEx::setAlternativeNavigationSystemEnabled(bool flag,bool forceConversion)	{
	if (flag==m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem && !forceConversion) return;
	m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem = flag;
	m_mouseButtons = -1;
	if (flag)	{
		// m_azi and m_ele to m_cameraT
		btMatrix3x3Helper::FromEulerYXZ(m_cameraT.getBasis(),btRadians(m_azi), btRadians(m_ele));
		m_cameraT.setOrigin(m_cameraTargetPosition-m_cameraT.getBasis().getColumn(2)*m_cameraDistance);
	}
	else	{
		// m_cameraT to m_azi and m_ele, m_cameraTargetPosition and m_cameraUp
		m_cameraPosition = m_cameraT.getOrigin();
		m_cameraTargetPosition = m_cameraPosition + m_cameraT.getBasis().getColumn(2) * m_cameraDistance;
		m_cameraUp.setValue(0,1,0);
		btMatrix3x3Helper::ToEulerYXZ(m_cameraT.getBasis(),m_azi, m_ele);
		m_azi=btDegrees(m_azi);
		m_ele=btDegrees(m_ele);			
	}
}	

void GlutDemoApplicationEx::calculateAndSetProjectionMatrix(int screenWidth,int screenHeight)	{
       if (screenWidth <= 0) screenWidth = m_glutScreenWidth;
       if (screenHeight <= 0) screenHeight = m_glutScreenHeight;
       if (screenWidth <= 0 || screenHeight <= 0) return;	
      
        		const btScalar aspect((btScalar)screenWidth / (btScalar)screenHeight);
                glMatrixMode(GL_PROJECTION);       		
                if (m_ortho)        Matrix16Helper::GetGluOrtho3DMatrix(m_cameraDistance,m_fovDeg,aspect,
        											-m_frustumZFar,	// To avoid clipping
        											//m_frustumZNear,
        											m_frustumZFar,m_projectionMatrix);	
        		else Matrix16Helper::GetGluPerspectiveMatrix(m_fovDeg,aspect,m_frustumZNear,m_frustumZFar,m_projectionMatrix);
        		#ifdef BT_USE_DOUBLE_PRECISION
        		glLoadMatrixd(m_projectionMatrix);
        		#else //BT_USE_DOUBLE_PRECISION
        		glLoadMatrixf(m_projectionMatrix);
        		#endif //BT_USE_DOUBLE_PRECISION
				//-------------------------------------------------------------

                glMatrixMode(GL_MODELVIEW);
                          

}

void GlutDemoApplicationEx::calculateAndSetModelviewMatrixFromCameraT()	{
        {
        // The "system" modelview matrix is the inverse of m_cameraT, BUT we must be coherent with the axis conventions...
        // If we choose +Z to be the "forward" axis, and +X the left axis (like we do), we must "spoil" the camera from its own convention,
        // because in openGL, if we assign an identity matrix to the camera, it looks in the -Z direction, and its +X direction
        // is "right", instead of left.
        // Many people force this convention (-Z = forward and +X = right) for all the objects, so they don't need to do this hack when
        // dealing with the camera, but I strongly advice you not to do so, as most models are modelled with the +Z axis as their
        // forward axis, and you'll end up rotating all the objects in the world, when you can just do some "dirty work" on the camera...
        //
        // P.S. For the ones that are still confused: don't worry, m_cameraT.getBasis().getColumn(2) is still the camera forward axis and that's
        // the important thing to remember.
        static btTransform m_cameraArrangedT;
        m_cameraArrangedT = m_cameraT;
        btMatrix3x3& m_cameraArrangedBasis = m_cameraArrangedT.getBasis();        
        //invert the sign of m_cameraArrangedT.getBasis().getColumn(2);	// Z axis
        m_cameraArrangedBasis[0].setZ(- m_cameraArrangedBasis[0][2]);
        m_cameraArrangedBasis[1].setZ(- m_cameraArrangedBasis[1][2]);
        m_cameraArrangedBasis[2].setZ(- m_cameraArrangedBasis[2][2]);
        //invert the sign of m_cameraArrangedT.getBasis().getColumn(0);  // X axis      
       	m_cameraArrangedBasis[0].setX( - m_cameraArrangedBasis[0][0]);
        m_cameraArrangedBasis[1].setX( - m_cameraArrangedBasis[1][0]);
        m_cameraArrangedBasis[2].setX( - m_cameraArrangedBasis[2][0]);        
        // Here m_cameraArrangedT has been cleaned from the "camera convention"
        m_cameraArrangedT.inverse().getOpenGLMatrix(m_modelviewMatrix);	// very fast (see the implementation of Matrix3x3::inverse())
        }
                                          
    glMatrixMode(GL_MODELVIEW);
	#ifdef BT_USE_DOUBLE_PRECISION
    glLoadMatrixd(m_modelviewMatrix);                                        	                     
    #else //BT_USE_DOUBLE_PRECISION
    glLoadMatrixf(m_modelviewMatrix);   
	#endif //BT_USE_DOUBLE_PRECISION	
}

void GlutDemoApplicationEx::refreshCameraRelatedVariables()	{
	if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem)	{
    	m_cameraPosition = m_cameraT.getOrigin();
        m_cameraUp = m_cameraT.getBasis().getColumn(1);
        m_cameraTargetPosition = m_cameraT.getOrigin() + m_cameraT.getBasis().getColumn(2)*m_cameraDistance;    // For backward compatibility
	}		
	else
	{

        btScalar rele = btRadians(m_ele);// rads per deg
        btScalar razi = btRadians(m_azi);// rads per deg


        btQuaternion rot(m_cameraUp,razi);


        btVector3 eyePos(0,0,0);
        eyePos[m_forwardAxis] = -m_cameraDistance;

        btVector3 forward(eyePos[0],eyePos[1],eyePos[2]);
        if (forward.length2() < SIMD_EPSILON)
        {
                forward.setValue(1.f,0.f,0.f);
        }
        btVector3 right = m_cameraUp.cross(forward);
        btQuaternion roll(right,-rele);

		m_cameraT.setBasis(btMatrix3x3(rot*roll));	// (btMatrix3x3(rot) * btMatrix3x3(roll));
		
        eyePos = m_cameraT.getBasis() * eyePos;

        m_cameraPosition[0] = eyePos.getX();
        m_cameraPosition[1] = eyePos.getY();
        m_cameraPosition[2] = eyePos.getZ();
        m_cameraPosition += m_cameraTargetPosition;

		m_cameraT.setOrigin(m_cameraPosition);
		        
	}	
}

void GlutDemoApplicationEx::updateCamera()
{
	refreshCameraRelatedVariables();

	if (m_ortho) calculateAndSetProjectionMatrix();
       
	calculateAndSetModelviewMatrixFromCameraT();      
}
static void EnableDefaultTextureSettings()	{
    glPushMatrix();
    	glMatrixMode(GL_TEXTURE);
        glLoadIdentity();
        glScalef(0.025f,0.025f,1.0f);//0.025f);
        glMatrixMode(GL_MODELVIEW);
    	glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		//glEnable(GL_TEXTURE_GEN_R);       
	glPopMatrix();					
}
static void DisableDefaultTextureSettings()	{
    glPushMatrix();
    	glMatrixMode(GL_TEXTURE);
        glLoadIdentity();
        
        glMatrixMode(GL_MODELVIEW);
    	glDisable(GL_TEXTURE_GEN_S);
        glDisable(GL_TEXTURE_GEN_T);
        //glDisable(GL_TEXTURE_GEN_R);        
	glPopMatrix();	
}


void GlutDemoApplicationEx::renderme()	{
 	glEnable(GL_COLOR_MATERIAL);
 	glEnable(GL_LIGHTING);
 	glEnable(GL_TEXTURE_2D);
 	//-------Default Texture Settings----------------------------
 	static const GLfloat	planex[]={1,0,0,0};
	static const GLfloat	planez[]={0,0,1,0};
	glTexGenfv(GL_S,GL_OBJECT_PLANE,planex);
	glTexGenfv(GL_T,GL_OBJECT_PLANE,planez);
	glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
	glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);		
	m_areDefaultTextureSettingsEnabled = (g_renderMode!=2);	
	if (m_areDefaultTextureSettingsEnabled) EnableDefaultTextureSettings();
	else DisableDefaultTextureSettings();
	//---------------------------------------------------------------
	glEnable(GL_LIGHTING);
	//glShadeModel(GL_SMOOTH);
	//glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	
	updateCamera();

	if (m_dynamicsWorld)
    {
        if (m_frustumCullingEnabled) performFrustumCulling();	// Frustum culling stuff
 
		if(m_enableshadows)
		{
			glClear(GL_STENCIL_BUFFER_BIT);
			glEnable(GL_CULL_FACE);
                        rendersceneEx(0);

			glDisable(GL_LIGHTING);
			glDepthMask(GL_FALSE);
			glDepthFunc(GL_LEQUAL);
			glEnable(GL_STENCIL_TEST);
			glColorMask(GL_FALSE,GL_FALSE,GL_FALSE,GL_FALSE);
			glStencilFunc(GL_ALWAYS,1,0xFFFFFFFFL);
			glFrontFace(GL_CCW);
			glStencilOp(GL_KEEP,GL_KEEP,GL_INCR);
			rendersceneEx(1);
			glFrontFace(GL_CW);
			glStencilOp(GL_KEEP,GL_KEEP,GL_DECR);
			rendersceneEx(1);
			glFrontFace(GL_CCW);

			glPolygonMode(GL_FRONT,GL_FILL);
			glPolygonMode(GL_BACK,GL_FILL);
			glShadeModel(GL_SMOOTH);
			glEnable(GL_DEPTH_TEST);
			glDepthFunc(GL_LESS);
			glEnable(GL_LIGHTING);
			glDepthMask(GL_TRUE);
			glCullFace(GL_BACK);
			glFrontFace(GL_CCW);
			glEnable(GL_CULL_FACE);
			glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);

			glDepthFunc(GL_LEQUAL);
			glStencilFunc( GL_NOTEQUAL, 0, 0xFFFFFFFFL );
			glStencilOp( GL_KEEP, GL_KEEP, GL_KEEP );
			glDisable(GL_LIGHTING);
			rendersceneEx(2);
			glEnable(GL_LIGHTING);
			glDepthFunc(GL_LESS);
			glDisable(GL_STENCIL_TEST);
			if (!g_cullBackFaces) glDisable(GL_CULL_FACE);
		}
		else
		{
			if (!g_cullBackFaces) glDisable(GL_CULL_FACE);
			else glEnable(GL_CULL_FACE);
			rendersceneEx(0);
		}

		int	xOffset = 10;
		int yStart = 20;
		int yIncr = 20;


		const bool mustShowRaytraceImage(m_showRaytraceImage && m_raytraceTextureId>0);				
		if (mustShowRaytraceImage && m_glutScreenHeight>0)
		{
           		glDisable(GL_LIGHTING);
				if (m_areDefaultTextureSettingsEnabled)	{
                	m_areDefaultTextureSettingsEnabled = false;
                	DisableDefaultTextureSettings();
                }
				glDisable(GL_DEPTH_TEST);
				glPushMatrix();
                	glLoadIdentity();
                	glTranslatef(0,0,-0.25f);
                	
                	const btScalar FOVTG=tan(m_fovDeg*1.5707963268f/180.0);	
                                        btScalar h = m_ortho ? 	m_cameraDistance*FOVTG : (m_frustumZNear+0.15f)*FOVTG;
                	btScalar w = h * btScalar(m_glutScreenWidth)/btScalar(m_glutScreenHeight);
                		
            	drawRaytraceImage(-0.3*w,-0.15*h,0.5*w,-0.95*h);
                glPopMatrix();
                glEnable(GL_DEPTH_TEST);
		}
			
		if ((m_debugMode & btIDebugDraw::DBG_NoHelpText)==0)
		{
			glColor3f(0, 0, 0);
			glDisable(GL_LIGHTING);
			//----------setOrthographicProjection()-----------------
			{
			glMatrixMode(GL_PROJECTION);
			glPushMatrix();
			glLoadIdentity();
			gluOrtho2D(0, m_glutScreenWidth, 0, m_glutScreenHeight);
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();			// New
			glLoadIdentity();
			// invert the y axis, down is positive
			glScalef(1, -1, 1);	
			// mover the origin from the bottom left corner
			// to the upper left corner
			glTranslatef(btScalar(0), btScalar(-m_glutScreenHeight), btScalar(0));		
			}
			//------------------------------------------------------	


			showProfileInfo(xOffset,yStart,yIncr);

#ifdef USE_QUICKPROF

		
			if ( getDebugMode() & btIDebugDraw::DBG_ProfileTimings)
			{
				static int counter = 0;
				counter++;
				std::map<std::string, hidden::ProfileBlock*>::iterator iter;
				for (iter = btProfiler::mProfileBlocks.begin(); iter != btProfiler::mProfileBlocks.end(); ++iter)
				{
					char blockTime[128];
					sprintf(blockTime, "%s: %lf",&((*iter).first[0]),btProfiler::getBlockTime((*iter).first, btProfiler::BLOCK_CYCLE_SECONDS));//BLOCK_TOTAL_PERCENT));
					glRasterPos3f(xOffset,yStart,0);
					GLDebugDrawString(BMF_GetFont(BMF_kHelvetica10),blockTime);
					yStart += yIncr;

				}

			}
#endif //USE_QUICKPROF


			//--------------resetPerspectiveProjection()---------------
			{
			glPopMatrix();
			glMatrixMode(GL_PROJECTION);
			glPopMatrix();
			glMatrixMode(GL_MODELVIEW);
			//updateCamera();	// no more
			}
			//---------------------------------------------------------

			//glEnable(GL_TEXTURE_2D);
		}
		
		//glEnable(GL_COLOR_MATERIAL);
 		glDisable(GL_LIGHTING);
	 	glDisable(GL_TEXTURE_2D);
	 	glColor3f(1,1,1);

	}
	
} 
void GlutDemoApplicationEx::rendersceneEx(int pass)	{
	btScalar	m[16];
	btMatrix3x3	rot;rot.setIdentity();
        const btAlignedObjectArray < btCollisionObject* >& collisionObjectArray = (m_frustumCullingEnabled) ? m_objectsInFrustum : m_dynamicsWorld->getCollisionObjectArray();
        const int	numObjects=collisionObjectArray.size();

        btVector3 wireColor(1,0,0);
	for(int i=0;i<numObjects;i++)
	{
        btCollisionObject*	colObj=collisionObjectArray[i];
        if (colObj==NULL || (pass==1 && colObj->getCollisionFlags()&CF_NO_SHADOW_CASTER) || (pass==2 && colObj->getCollisionFlags()&CF_NO_SHADOW_RECEIVER)) continue;	

        
		// New: -------------------------------------------------
		#ifdef USE_COLLISION_OBJECT_USER_POINTER
		if (g_renderMode%3==2) 	{
		btCollisionObjectUserPointer* up = static_cast < btCollisionObjectUserPointer* > (colObj->getUserPointer());
		if (up)	{
			btCollisionShape* shape = const_cast < btCollisionShape* > (colObj->getCollisionShape());
			if (shape)	{
				GL_MyShapeDrawer::MyShapeCache*	mysc = static_cast <GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(shape);
				if (mysc)	{
					mysc->customMeshGuest = up->customMesh;	// these 2 are deleted soon after the rendering
					mysc->overridesGuest = &up->overrides;
				}	
			}
		}	
		}
		#endif //USE_COLLISION_OBJECT_USER_POINTER		
		// ------------------------------------------------------
		btRigidBody*		body=btRigidBody::upcast(colObj);
		if(body&&body->getMotionState())
		{
			bool useCOMoffset(false);				
			#ifdef USE_COLLISION_OBJECT_USER_POINTER
			if (g_renderMode%3==2 && pass!=1) 	{
				const btCollisionShape* shape = colObj->getCollisionShape();
				if (shape && shape->getUserPointer()) {
					const GL_MyShapeDrawer::MyShapeCache* mysc = static_cast < const GL_MyShapeDrawer::MyShapeCache* > (shape->getUserPointer());		
                                        //const btTriangleMeshExFootPrint* customMesh = mysc->customMesh;
					const bool drawCustomMesh = mysc->customMesh || mysc->customMeshGuest;
					if (drawCustomMesh)	{
						btMyDefaultMotionState* myMotionState = btMyDefaultMotionState::upcast(body);	// For unknown reasons this returns always NULL AFAIK
																										// and for more unknown reasons there's no gap between graphics and physics... (a gap should exist if COM!=0)
						if (myMotionState)	{
							btTransform T;
							myMotionState->getWorldTransformOutsideCOM(T);								
							T.getOpenGLMatrix(m);
							rot = T.getBasis();
							useCOMoffset = true;
							//printf("OK ");
						}
						//else printf("!OK1 ");	
					}
					//else printf("!OK2 ");
				}
			}								
			#endif //USE_COLLISION_OBJECT_USER_POINTER
			if (!useCOMoffset)	{
				btDefaultMotionState* myMotionState = (btDefaultMotionState*)body->getMotionState();
				myMotionState->m_graphicsWorldTrans.getOpenGLMatrix(m);
				rot=myMotionState->m_graphicsWorldTrans.getBasis();
			}	
		}
		else
		{
			colObj->getWorldTransform().getOpenGLMatrix(m);
			rot=colObj->getWorldTransform().getBasis();
		}
                #define USE_ALTERNATIVE_COLORS	// switchable, but colors depend on 'i' otherwise, so it works only if the object array is always of the same size with the same order (not good for frustum culling)
                #ifndef USE_ALTERNATIVE_COLORS
                btVector3 wireColor(0.f,0.0f,1.f); //wants deactivation
                if(i&1) wireColor=btVector3(1.f,1.0f,0.5f);
                ///color differently for active, sleeping, wantsdeactivation states
                if (colObj->getActivationState() == 1) //active
                {
                if (i & 1)
                    {
                     wireColor += btVector3 (.5f,0.f,0.f);
                    }
                else
                    {
                     wireColor += btVector3 (1.f,0.f,0.f);
                    }
                }
                if(colObj->getActivationState()==2) //ISLAND_SLEEPING
                    {
                    if(i&1)
                        {
                        wireColor += btVector3 (0.f,0.5f,0.f);
                        }
                    else
                        {
                        wireColor += btVector3 (0.f,1.f, 0.f);
                        }
                    }
                #else //USE_ALTERNATIVE_COLORS
                // code posted by "orange" to the Bullet Forum:
                btVector3 wireColor(btScalar(1),btScalar(1),btScalar(0.4));
                switch(colObj->getActivationState())
                {
                    case  ACTIVE_TAG:
                    wireColor = btVector3(btScalar(1),btScalar(1),btScalar(0.4)); break;
                    case ISLAND_SLEEPING:
                    wireColor =  btVector3(btScalar(0.),btScalar(1),btScalar(0.));break;
                    case WANTS_DEACTIVATION:
                    wireColor = btVector3(btScalar(0.),btScalar(1),btScalar(1));break;
                    case DISABLE_DEACTIVATION:
                    wireColor = btVector3(btScalar(1),btScalar(0.),btScalar(0.));break;
                    case DISABLE_SIMULATION:
                    wireColor = btVector3(btScalar(1),btScalar(1),btScalar(0.));break;
                    default:
                    {
                    wireColor = btVector3(btScalar(1),btScalar(0.),btScalar(0.));
                    }
                }
                #undef USE_ALTERNATIVE_COLORS
                #endif //USE_ALTERNATIVE_COLORS

		btVector3 aabbMin,aabbMax;
		m_dynamicsWorld->getBroadphase()->getBroadphaseAabb(aabbMin,aabbMax);
		
		aabbMin-=btVector3(BT_LARGE_FLOAT,BT_LARGE_FLOAT,BT_LARGE_FLOAT);
		aabbMax+=btVector3(BT_LARGE_FLOAT,BT_LARGE_FLOAT,BT_LARGE_FLOAT);
//		printf("aabbMin=(%f,%f,%f)\n",aabbMin.getX(),aabbMin.getY(),aabbMin.getZ());
//		printf("aabbMax=(%f,%f,%f)\n",aabbMax.getX(),aabbMax.getY(),aabbMax.getZ());
//		m_dynamicsWorld->getDebugDrawer()->drawAabb(aabbMin,aabbMax,btVector3(1,1,1));


		if (!(getDebugMode()& btIDebugDraw::DBG_DrawWireframe))
		{
			switch(pass)
			{
			case	0:	static_cast <GL_MyShapeDrawer*> (m_shapeDrawer)->drawOpenGL(m,colObj->getCollisionShape(),wireColor,getDebugMode(),aabbMin,aabbMax,pass);break;
			case	1:	static_cast <GL_MyShapeDrawer*> (m_shapeDrawer)->drawShadow(m,m_sundirection*rot,colObj->getCollisionShape(),aabbMin,aabbMax);break;
			case	2:	static_cast <GL_MyShapeDrawer*> (m_shapeDrawer)->drawOpenGL(m,colObj->getCollisionShape(),wireColor*btScalar(0.3),0,aabbMin,aabbMax,pass);break;
			}
		}
	}	
}


#ifdef USE_COLLISION_OBJECT_USER_POINTER
btCollisionObjectUserPointer::btCollisionObjectUserPointer() : customMesh(NULL), userPointer(NULL)	{}
btCollisionObjectUserPointer::~btCollisionObjectUserPointer()	{
	//if (customMesh) btTriangleMeshEx::DeleteMeshFootPrint(customMesh);
}
void GlutDemoApplicationEx::AssignMesh(btCollisionObject* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling, const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArray,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray)	{
	if (!shape) return;
	//printf("void GlutDemoApplicationEx::AssignMesh(...)\n");	
	btCollisionObjectUserPointer* mysc = btCollisionObjectUserPointer::GetFrom(shape);
	if (!mysc) return;
	mesh.createMeshFootPrint(mysc->customMesh,scaling,useFlatNormals,pOptionalMaterialOverrideArray,pOptionalTextureOverrideArray,pOptionalSubmeshIndexArray);
	mysc->overrides.reset();
	//printf("End void GlutDemoApplicationEx::AssignMesh(...)\n");
}
void GlutDemoApplicationEx::AssignMesh(btCollisionObject* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride)	{
	btAlignedObjectArray< const Material* > optionalMaterialOverrideArray;
	optionalMaterialOverrideArray.resize(1);
	optionalMaterialOverrideArray[0] = &firstMaterialOverride;	// it should be OK to pass ptrs happily this way: the material is deep cloned.
        AssignMesh(shape,mesh,useFlatNormals,scaling,&optionalMaterialOverrideArray);
}
void GlutDemoApplicationEx::AssignMesh(btCollisionObject* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const GLuint firstTextureIdOverride)	{
	btAlignedObjectArray< GLuint > optionalTextureOverrideArray;
	optionalTextureOverrideArray.resize(1);
	optionalTextureOverrideArray[0] = firstTextureIdOverride;	
        AssignMesh(shape,mesh,useFlatNormals,scaling,NULL,&optionalTextureOverrideArray);
}
void GlutDemoApplicationEx::AssignMesh(btCollisionObject* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride,const GLuint firstTextureIdOverride)	{
	btAlignedObjectArray< const Material* > optionalMaterialOverrideArray;
	optionalMaterialOverrideArray.resize(1);
	optionalMaterialOverrideArray[0] = &firstMaterialOverride;	// it should be OK to pass ptrs happily this way: the material is deep cloned.
	btAlignedObjectArray< GLuint > optionalTextureOverrideArray;
	optionalTextureOverrideArray.resize(1);
	optionalTextureOverrideArray[0] = firstTextureIdOverride;
        AssignMesh(shape,mesh,useFlatNormals,scaling,&optionalMaterialOverrideArray,&optionalTextureOverrideArray);
}
void GlutDemoApplicationEx::AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionShape* shapeFrom)	{
	if (!colObjTo || !shapeFrom) return;
	btCollisionObjectUserPointer* myscTo = btCollisionObjectUserPointer::GetFrom(colObjTo);
	const GL_MyShapeDrawer::MyShapeCache*	myscFrom = const_cast < const GL_MyShapeDrawer::MyShapeCache* > (static_cast < GL_MyShapeDrawer* > (this->m_shapeDrawer)->mycache(const_cast < btCollisionShape*> (shapeFrom)));
	if (!myscTo || !myscFrom) return;
	myscTo->customMesh = myscFrom->customMesh;
	myscTo->overrides = myscFrom->overrides;
}
void GlutDemoApplicationEx::AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionObject* colObjFrom)	{
	if (!colObjTo || !colObjFrom) return;
	btCollisionObjectUserPointer* myscTo = btCollisionObjectUserPointer::GetFrom(colObjTo);
	const btCollisionObjectUserPointer* myscFrom = btCollisionObjectUserPointer::GetFrom(colObjFrom);
	if (!myscTo || !myscFrom) return;
	myscTo->customMesh = myscFrom->customMesh;
	myscTo->overrides = myscFrom->overrides;
}


const btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(const btCollisionObject& co) const	{
	btCollisionObjectUserPointer* p = btCollisionObjectUserPointer::GetFrom(&co);
	return p->overrides;
}
btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(btCollisionObject& co)	{
	btCollisionObjectUserPointer* p = btCollisionObjectUserPointer::GetFrom(&co);
	return p->overrides;	
}

btCollisionObjectUserPointer* btCollisionObjectUserPointer::GetFrom(const btCollisionObject* obj)	{
	if (!obj) return NULL;
	btCollisionObject* o = const_cast < btCollisionObject* >(obj);
	btCollisionObjectUserPointer* p = (btCollisionObjectUserPointer*) obj->getUserPointer();
	if (!p) {
		p = new btCollisionObjectUserPointer();
		o->setUserPointer(p);
	}	
	return p;
}
void btCollisionObjectUserPointer::SafeDeleteUserPointer(btCollisionObject* obj)	{
	if (!obj) return;
	btCollisionObjectUserPointer* p = btCollisionObjectUserPointer::GetFrom(obj);
	if (p) {
		delete p;p = NULL;
		obj->setUserPointer(NULL);
	}
}

btMyDefaultMotionState::~btMyDefaultMotionState()	{
}
btMyDefaultMotionState* btMyDefaultMotionState::upcast(btRigidBody* co) {
	return (co ? dynamic_cast < btMyDefaultMotionState* >(co->getMotionState()) : NULL);
}
const btMyDefaultMotionState* btMyDefaultMotionState::upcast(const btRigidBody* co) {
	return (co ? dynamic_cast < const btMyDefaultMotionState* >(co->getMotionState()) : NULL);
}
btMyDefaultMotionState::btMyDefaultMotionState(const btTransform& startTrans,const btVector3& _centerOfMassOffset)
 : btDefaultMotionState(btTransform(startTrans.getBasis(),startTrans.getOrigin()+startTrans.getBasis()*_centerOfMassOffset),btTransform::getIdentity()),centerOfMassOffset(_centerOfMassOffset)	{

}
void btMyDefaultMotionState::getWorldTransformOutsideCOM(btTransform& worldTransOutsideCOM) const	{
		btTransform centerOfMassWorldTrans;
		getWorldTransform( centerOfMassWorldTrans ); 
		worldTransOutsideCOM.setBasis(centerOfMassWorldTrans.getBasis());
		worldTransOutsideCOM.setOrigin(centerOfMassWorldTrans.getOrigin()-centerOfMassWorldTrans.getBasis()*centerOfMassOffset);				
}
void btMyDefaultMotionState::setWorldTransformOutsideCOM(const btTransform& worldTransOutsideCOM)	{
		btTransform worldTransInCOM(worldTransOutsideCOM.getBasis(),worldTransOutsideCOM.getOrigin()+worldTransOutsideCOM.getBasis()*centerOfMassOffset);
		setWorldTransform(worldTransInCOM);
}
void	btMyDefaultMotionState::getWorldTransform(btTransform& centerOfMassWorldTrans ) const {
                        centerOfMassWorldTrans =  m_graphicsWorldTrans ;
}
void	btMyDefaultMotionState::setWorldTransform(const btTransform& centerOfMassWorldTrans){
                        m_graphicsWorldTrans = centerOfMassWorldTrans;
}


#endif //USE_COLLISION_OBJECT_USER_POINTER

void GlutDemoApplicationEx::SetUserPointer(btCollisionObject* shape,void* userPointer)	{
    if (!shape) return;
	#ifdef USE_COLLISION_OBJECT_USER_POINTER
	btCollisionObjectUserPointer* mysc = btCollisionObjectUserPointer::GetFrom(shape);
	if (!mysc) return;
	mysc->userPointer = userPointer;
	#else //USE_COLLISION_OBJECT_USER_POINTER
	shape->setUserPointer(userPointer);
	#endif //USE_COLLISION_OBJECT_USER_POINTER
}
void* GlutDemoApplicationEx::GetUserPointer(const btCollisionObject* shape)	{
	if (!shape || !shape->getUserPointer()) return NULL;
	#ifdef USE_COLLISION_OBJECT_USER_POINTER	
	btCollisionObjectUserPointer* mysc = btCollisionObjectUserPointer::GetFrom(shape);
	if (!mysc) return NULL;
	return mysc->userPointer;
	#else //USE_COLLISION_OBJECT_USER_POINTER
	return shape->getUserPointer();
	#endif //USE_COLLISION_OBJECT_USER_POINTER
}



void GL_MyShapeDrawer::drawOpenGL(btScalar* m, const btCollisionShape* shape, const btVector3& color,int	debugMode,const btVector3& worldBoundsMin,const btVector3& worldBoundsMax,const int pass)	{
	if (GlutDemoApplicationEx::g_renderMode%3==2 && shape->getUserPointer()) {
		const MyShapeCache* mysc = static_cast < const MyShapeCache* > (shape->getUserPointer());		
                const bool drawCustomMesh = mysc->customMesh || mysc->customMeshGuest;
		if (drawCustomMesh)	{
		
		if (*m_areDefaultTextureSettingsEnabled) {
			*m_areDefaultTextureSettingsEnabled = false;
			DisableDefaultTextureSettings();
		}
		

        const bool useAmbientColorOnly(pass==2);
        if (useAmbientColorOnly) glEnable(GL_COLOR_MATERIAL);	
        else glDisable(GL_COLOR_MATERIAL);
        glPushMatrix();
			btglMultMatrix(m);
			if (mysc->customMeshGuest) mysc->customMeshGuest->glDraw(mysc->overridesGuest,NULL,useAmbientColorOnly);	
			else if (mysc->customMesh) mysc->customMesh->glDraw(mysc->overridesGuest,&mysc->overrides,useAmbientColorOnly);
		glPopMatrix();
		
		// Reset materials ?				
		//Material().bind();
		 glEnable(GL_COLOR_MATERIAL);
		
		//glColor3f(color.x(),color.y(), color.z());
		//if(m_textureenabled&&(m_textureinitialized)) glBindTexture(GL_TEXTURE_2D,m_texturehandle);
		
		//-------------------
		//printf("%s ",shape->getName());
		}
		// Remove guest stuff--------
			MyShapeCache* myncsc = const_cast < MyShapeCache* > (mysc);				
			myncsc->customMeshGuest= NULL;
			myncsc->overridesGuest = NULL;			
		//---------------------------
		if (drawCustomMesh) return;
	}		
	
	if (shape->getShapeType() == CUSTOM_CONVEX_SHAPE_TYPE)
	{
		btVector3 org(m[12], m[13], m[14]);
		btVector3 dx(m[0], m[1], m[2]);
		btVector3 dy(m[4], m[5], m[6]);
//		btVector3 dz(m[8], m[9], m[10]);
		const btBoxShape* boxShape = static_cast<const btBoxShape*>(shape);
		btVector3 halfExtent = boxShape->getHalfExtentsWithMargin();
		dx *= halfExtent[0];
		dy *= halfExtent[1];
//		dz *= halfExtent[2];
		glColor3f(1,1,1);
		glDisable(GL_LIGHTING);
		glLineWidth(2);

		glBegin(GL_LINE_LOOP);
		glDrawVector(org - dx - dy);
		glDrawVector(org - dx + dy);
		glDrawVector(org + dx + dy);
		glDrawVector(org + dx - dy);
		glEnd();
		return;
	} 
	else if((shape->getShapeType() == BOX_SHAPE_PROXYTYPE) && (debugMode & btIDebugDraw::DBG_FastWireframe))
	{
		btVector3 org(m[12], m[13], m[14]);
		btVector3 dx(m[0], m[1], m[2]);
		btVector3 dy(m[4], m[5], m[6]);
		btVector3 dz(m[8], m[9], m[10]);
		const btBoxShape* boxShape = static_cast<const btBoxShape*>(shape);
		btVector3 halfExtent = boxShape->getHalfExtentsWithMargin();
		dx *= halfExtent[0];
		dy *= halfExtent[1];
		dz *= halfExtent[2];
		glBegin(GL_LINE_LOOP);
		glDrawVector(org - dx - dy - dz);
		glDrawVector(org + dx - dy - dz);
		glDrawVector(org + dx + dy - dz);
		glDrawVector(org - dx + dy - dz);
		glDrawVector(org - dx + dy + dz);
		glDrawVector(org + dx + dy + dz);
		glDrawVector(org + dx - dy + dz);
		glDrawVector(org - dx - dy + dz);
		glEnd();
		glBegin(GL_LINES);
		glDrawVector(org + dx - dy - dz);
		glDrawVector(org + dx - dy + dz);
		glDrawVector(org + dx + dy - dz);
		glDrawVector(org + dx + dy + dz);
		glDrawVector(org - dx - dy - dz);
		glDrawVector(org - dx + dy - dz);
		glDrawVector(org - dx - dy + dz);
		glDrawVector(org - dx + dy + dz);
		glEnd();
		return;
	}

	glPushMatrix(); 
	btglMultMatrix(m);


	if (shape->getShapeType() == UNIFORM_SCALING_SHAPE_PROXYTYPE)
	{
		const btUniformScalingShape* scalingShape = static_cast<const btUniformScalingShape*>(shape);
		const btConvexShape* convexShape = scalingShape->getChildShape();
		float	scalingFactor = (float)scalingShape->getUniformScalingFactor();
		{
			btScalar tmpScaling[4][4]={{scalingFactor,0,0,0},
			{0,scalingFactor,0,0},
			{0,0,scalingFactor,0},
			{0,0,0,1}};

			drawOpenGL( (btScalar*)tmpScaling,convexShape,color,debugMode,worldBoundsMin,worldBoundsMax);
		}
		glPopMatrix();
		return;
	}

	if (shape->getShapeType() == COMPOUND_SHAPE_PROXYTYPE)
	{
		const btCompoundShape* compoundShape = static_cast<const btCompoundShape*>(shape);
		const static btVector3 colors[] = {
			btVector3(1,1,1),
			btVector3(1,0,0),
			btVector3(0,1,0),
			btVector3(0,0,1),
			btVector3(1,1,0),
			btVector3(0.8,0.4,0.8),
			btVector3(0.5,0.5,0.5),
			btVector3(0.45,0.65,0.75),
			btVector3(0.5,0.2,0),
			btVector3(0,0.5,0),
			btVector3(0,0,0.5),
			btVector3(0.75,0.75,0.25),
			btVector3(0.75,0.5,0.25),
			btVector3(0.4,0.4,0.4),
			btVector3(0.4,0,0),
			btVector3(0,0.4,0),
			btVector3(0,0,0.4),
			btVector3(0.4,0.4,0),
			btVector3(0.4,0.2,0)			
		};
		static const int colorsSize = sizeof(colors)/sizeof(colors[0]);
		for (int i=compoundShape->getNumChildShapes()-1;i>=0;i--)
		{
			btTransform childTrans = compoundShape->getChildTransform(i);
			const btCollisionShape* colShape = compoundShape->getChildShape(i);
			btScalar childMat[16];
			childTrans.getOpenGLMatrix(childMat);
			drawOpenGL(childMat,colShape,GlutDemoApplicationEx::g_colorMode ? colors[i%colorsSize] : color,debugMode,worldBoundsMin,worldBoundsMax);
		}	
	} else
	{
		if(m_textureenabled&&(!m_textureinitialized))
		{
			GLubyte*	image=new GLubyte[256*256*3];
			for(int y=0;y<256;++y)
			{
				const int	t=y>>4;
				GLubyte*	pi=image+y*256*3;
				for(int x=0;x<256;++x)
				{
					const int		s=x>>4;
					const GLubyte	b=180;					
					GLubyte			c=b+((s+t&1)&1)*(255-b);
					pi[0]=pi[1]=pi[2]=c;pi+=3;
				}
			}

			glGenTextures(1,(GLuint*)&m_texturehandle);
			glBindTexture(GL_TEXTURE_2D,m_texturehandle);
			glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
			glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
			glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
			glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
			glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
			gluBuild2DMipmaps(GL_TEXTURE_2D,3,256,256,GL_RGB,GL_UNSIGNED_BYTE,image);
			delete[] image;
	
			m_textureinitialized=true;
		}			

		
		//drawCoordSystem();

		//glPushMatrix();
		//glEnable(GL_COLOR_MATERIAL);
		if(m_textureenabled) 
		{
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D,m_texturehandle);
			if (!*m_areDefaultTextureSettingsEnabled) {
				*m_areDefaultTextureSettingsEnabled = true;
				EnableDefaultTextureSettings();
			}	
		} else
		{
			glDisable(GL_TEXTURE_2D);
		}


                glColor3f(color.x(),color.y(), color.z());

                bool useWireframeFallback = true;

		if (!(debugMode & btIDebugDraw::DBG_DrawWireframe))
		{
			///you can comment out any of the specific cases, and use the default

			///the benefit of 'default' is that it approximates the actual collision shape including collision margin
			//int shapetype=m_textureenabled?MAX_BROADPHASE_COLLISION_TYPES:shape->getShapeType();
			int shapetype=shape->getShapeType();
			switch (shapetype)
			{

				case SPHERE_SHAPE_PROXYTYPE:
				{
					const btSphereShape* sphereShape = static_cast<const btSphereShape*>(shape);
					float radius = sphereShape->getMargin();//radius doesn't include the margin, so draw with margin
					drawSphere(radius,10,10);
					useWireframeFallback = false;
					break;
				}

				case BOX_SHAPE_PROXYTYPE:
				{
					const btBoxShape* boxShape = static_cast<const btBoxShape*>(shape);
					btVector3 halfExtent = boxShape->getHalfExtentsWithMargin();
					
					static int indices[36] = {
						0,1,2,
						3,2,1,
						4,0,6,
						6,0,2,
						5,1,4,
						4,1,0,
						7,3,1,
						7,1,5,
						5,4,7,
						7,4,6,
						7,2,3,
						7,6,2};

					 btVector3 vertices[8]={	
						btVector3(halfExtent[0],halfExtent[1],halfExtent[2]),
						btVector3(-halfExtent[0],halfExtent[1],halfExtent[2]),
						btVector3(halfExtent[0],-halfExtent[1],halfExtent[2]),	
						btVector3(-halfExtent[0],-halfExtent[1],halfExtent[2]),	
						btVector3(halfExtent[0],halfExtent[1],-halfExtent[2]),
						btVector3(-halfExtent[0],halfExtent[1],-halfExtent[2]),	
						btVector3(halfExtent[0],-halfExtent[1],-halfExtent[2]),	
						btVector3(-halfExtent[0],-halfExtent[1],-halfExtent[2])};
#if 1
					glBegin (GL_TRIANGLES);
					int si=36;
					for (int i=0;i<si;i+=3)
					{
						const btVector3& v1 = vertices[indices[i]];;
						const btVector3& v2 = vertices[indices[i+1]];
						const btVector3& v3 = vertices[indices[i+2]];
						btVector3 normal = (v3-v1).cross(v2-v1);
						normal.normalize ();
						glNormal3f(normal.getX(),normal.getY(),normal.getZ());
						glVertex3f (v1.x(), v1.y(), v1.z());
						glVertex3f (v2.x(), v2.y(), v2.z());
						glVertex3f (v3.x(), v3.y(), v3.z());
						
					}
					glEnd();
#endif

					useWireframeFallback = false;
					break;
				}



#if 0
			
			case CONE_SHAPE_PROXYTYPE:
				{
					const btConeShape* coneShape = static_cast<const btConeShape*>(shape);
					int upIndex = coneShape->getConeUpIndex();
					float radius = coneShape->getRadius();//+coneShape->getMargin();
					float height = coneShape->getHeight();//+coneShape->getMargin();
					switch (upIndex)
					{
					case 0:
						glRotatef(90.0, 0.0, 1.0, 0.0);
						break;
					case 1:
						glRotatef(-90.0, 1.0, 0.0, 0.0);
						break;
					case 2:
						break;
					default:
						{
						}
					};

					glTranslatef(0.0, 0.0, -0.5*height);
					glutSolidCone(radius,height,10,10);
					useWireframeFallback = false;
					break;

				}
#endif

			case STATIC_PLANE_PROXYTYPE:
				{
					const btStaticPlaneShape* staticPlaneShape = static_cast<const btStaticPlaneShape*>(shape);
					btScalar planeConst = staticPlaneShape->getPlaneConstant();
					const btVector3& planeNormal = staticPlaneShape->getPlaneNormal();
					btVector3 planeOrigin = planeNormal * planeConst;
					btVector3 vec0,vec1;
					btPlaneSpace1(planeNormal,vec0,vec1);
					btScalar vecLen = 100.f;
					btVector3 pt0 = planeOrigin + vec0*vecLen;
					btVector3 pt1 = planeOrigin - vec0*vecLen;
					btVector3 pt2 = planeOrigin + vec1*vecLen;
					btVector3 pt3 = planeOrigin - vec1*vecLen;
					glBegin(GL_LINES);
					glVertex3f(pt0.getX(),pt0.getY(),pt0.getZ());
					glVertex3f(pt1.getX(),pt1.getY(),pt1.getZ());
					glVertex3f(pt2.getX(),pt2.getY(),pt2.getZ());
					glVertex3f(pt3.getX(),pt3.getY(),pt3.getZ());
					glEnd();


					break;

				}

/*
			case CYLINDER_SHAPE_PROXYTYPE:
				{
					const btCylinderShape* cylinder = static_cast<const btCylinderShape*>(shape);
					int upAxis = cylinder->getUpAxis();


					float radius = cylinder->getRadius();
					float halfHeight = cylinder->getHalfExtentsWithMargin()[upAxis];

					drawCylinder(radius,halfHeight,upAxis);

					break;
				}
*/

			case MULTI_SPHERE_SHAPE_PROXYTYPE:
			{
				const btMultiSphereShape* multiSphereShape = static_cast<const btMultiSphereShape*>(shape);

				btTransform childTransform;
				childTransform.setIdentity();

				
				for (int i = multiSphereShape->getSphereCount()-1; i>=0;i--)
				{
					btSphereShape sc(multiSphereShape->getSphereRadius(i));
					childTransform.setOrigin(multiSphereShape->getSpherePosition(i));
					btScalar childMat[16];
					childTransform.getOpenGLMatrix(childMat);
					drawOpenGL(childMat,&sc,color,debugMode,worldBoundsMin,worldBoundsMax);
				}

				break;
			}

			default:
				{
					if (shape->isConvex())
					{
						const btConvexPolyhedron* poly = shape->isPolyhedral() ? ((btPolyhedralConvexShape*) shape)->getConvexPolyhedron() : 0;
						if (poly)
						{
							int i;
							glBegin (GL_TRIANGLES);
							for (i=0;i<poly->m_faces.size();i++)
							{
								btVector3 centroid(0,0,0);
								int numVerts = poly->m_faces[i].m_indices.size();
								if (numVerts>2)
								{
									btVector3 v1 = poly->m_vertices[poly->m_faces[i].m_indices[0]];
									for (int v=0;v<poly->m_faces[i].m_indices.size()-2;v++)
									{
										
										btVector3 v2 = poly->m_vertices[poly->m_faces[i].m_indices[v+1]];
										btVector3 v3 = poly->m_vertices[poly->m_faces[i].m_indices[v+2]];
										btVector3 normal = (v3-v1).cross(v2-v1);
										normal.normalize ();
										glNormal3f(normal.getX(),normal.getY(),normal.getZ());
										glVertex3f (v1.x(), v1.y(), v1.z());
										glVertex3f (v2.x(), v2.y(), v2.z());
										glVertex3f (v3.x(), v3.y(), v3.z());
									}
								}
							}
							glEnd ();
						} else
						{
							MyShapeCache*	mysc=mycache((btCollisionShape*)shape);
							GLuint dl = GlutDemoApplicationEx::g_renderMode%3;
                                                        //dl = dl<2 ? mysc->dl[dl] : 0;
                                                        dl = dl<2 ? mysc->dl[dl] : mysc->dl[1];
							if (dl==0) dl = mysc->dl[0];
							glCallList(dl);
						}
					}
				}
			}

		}


		glNormal3f(0,1,0);
	

		/// for polyhedral shapes
		if (debugMode==btIDebugDraw::DBG_DrawFeaturesText && (shape->isPolyhedral()))
		{
			btPolyhedralConvexShape* polyshape = (btPolyhedralConvexShape*) shape;

			{

				glColor3f(1.f, 1.f, 1.f);
				int i;
				for (i=0;i<polyshape->getNumVertices();i++)
				{
					btVector3 vtx;
					polyshape->getVertex(i,vtx);
					char buf[12];
					sprintf(buf," %d",i);
					//btDrawString(BMF_GetFont(BMF_kHelvetica10),buf);
				}

				for (i=0;i<polyshape->getNumPlanes();i++)
				{
					btVector3 normal;
					btVector3 vtx;
					polyshape->getPlane(normal,vtx,i);
					//btScalar d = vtx.dot(normal);

					//char buf[12];
					//sprintf(buf," plane %d",i);
					//btDrawString(BMF_GetFont(BMF_kHelvetica10),buf);

				}
			}

		}


#ifdef USE_DISPLAY_LISTS

		if (shape->getShapeType() == TRIANGLE_MESH_SHAPE_PROXYTYPE||shape->getShapeType() == GIMPACT_SHAPE_PROXYTYPE)
		{
			GLuint dlist =   OGL_get_displaylist_for_shape((btCollisionShape * )shape);
			if (dlist)
			{
				glCallList(dlist);
			}
			else
			{
#else		
		if (shape->isConcave() && !shape->isInfinite())
		{
			btConcaveShape* concaveMesh = (btConcaveShape*) shape;

			GlDrawcallback drawCallback;
			drawCallback.m_wireframe = (debugMode & btIDebugDraw::DBG_DrawWireframe)!=0;

			concaveMesh->processAllTriangles(&drawCallback,worldBoundsMin,worldBoundsMax);

		}
#endif

#ifdef USE_DISPLAY_LISTS
	}
}
#endif


		//glColor3f(0.4f,0.4f,0.4f);		



	}
	glPopMatrix();

}
void GL_MyShapeDrawer::drawShadow(btScalar* m,const btVector3& extrusion,const btCollisionShape* shape,const btVector3& worldBoundsMin,const btVector3& worldBoundsMax)	{
        glColor3fv(&GlutDemoApplicationEx::g_shadowColor.x());
		
	glPushMatrix(); 
	btglMultMatrix(m);
	if(shape->getShapeType() == UNIFORM_SCALING_SHAPE_PROXYTYPE)
	{
		const btUniformScalingShape* scalingShape = static_cast<const btUniformScalingShape*>(shape);
		const btConvexShape* convexShape = scalingShape->getChildShape();
		float	scalingFactor = (float)scalingShape->getUniformScalingFactor();
		btScalar tmpScaling[4][4]={	{scalingFactor,0,0,0},
		{0,scalingFactor,0,0},
		{0,0,scalingFactor,0},
		{0,0,0,1}};
		drawShadow((btScalar*)tmpScaling,extrusion,convexShape,worldBoundsMin,worldBoundsMax);
		glPopMatrix();
		return;
	}
	else if(shape->getShapeType()==COMPOUND_SHAPE_PROXYTYPE)
	{
		const btCompoundShape* compoundShape = static_cast<const btCompoundShape*>(shape);
		for (int i=compoundShape->getNumChildShapes()-1;i>=0;i--)
		{
			btTransform childTrans = compoundShape->getChildTransform(i);
			const btCollisionShape* colShape = compoundShape->getChildShape(i);
			btScalar childMat[16];
			childTrans.getOpenGLMatrix(childMat);
			drawShadow(childMat,extrusion*childTrans.getBasis(),colShape,worldBoundsMin,worldBoundsMax);
		}
	}
	else
	{
	//	bool useWireframeFallback = true;
		if (shape->isConvex())
		{
			const MyShapeCache*	mysc=mycache((btConvexShape*)shape);
			const ShapeCache* sc = &mysc->shapeCache;
			const btShapeHull* hull = &sc->m_shapehull;
			glBegin(GL_QUADS);
			for(int i=0;i<sc->m_edges.size();++i)
			{			
				const btScalar		d=btDot(sc->m_edges[i].n[0],extrusion);
				if((d*btDot(sc->m_edges[i].n[1],extrusion))<0)
				{
					const int			q=	d<0?1:0;
					const btVector3&	a=	hull->getVertexPointer()[sc->m_edges[i].v[q]];
					const btVector3&	b=	hull->getVertexPointer()[sc->m_edges[i].v[1-q]];
					glVertex3f(a[0],a[1],a[2]);
					glVertex3f(b[0],b[1],b[2]);
					glVertex3f(b[0]+extrusion[0],b[1]+extrusion[1],b[2]+extrusion[2]);
					glVertex3f(a[0]+extrusion[0],a[1]+extrusion[1],a[2]+extrusion[2]);
				}
			}
			glEnd();
		}

	}




	if (shape->isConcave())//>getShapeType() == TRIANGLE_MESH_SHAPE_PROXYTYPE||shape->getShapeType() == GIMPACT_SHAPE_PROXYTYPE)
		//		if (shape->getShapeType() == TRIANGLE_MESH_SHAPE_PROXYTYPE)
	{
		btConcaveShape* concaveMesh = (btConcaveShape*) shape;

		GlDrawcallback drawCallback;
		drawCallback.m_wireframe = false;

		concaveMesh->processAllTriangles(&drawCallback,worldBoundsMin,worldBoundsMax);

	}
	glPopMatrix();

}


// Picking Stuff----------------------------------------------------------
extern int gPickingConstraintId;
extern btVector3 gOldPickingPos;
extern btVector3 gHitPos;
extern float gOldPickingDist;
extern btRigidBody* pickedBody;
extern btScalar mousePickClamping;
static bool use6Dof = false;

btVector3	GlutDemoApplicationEx::getRayToEx(int x,int y)
{

        const btScalar aspect = (btScalar) m_glutScreenWidth/(btScalar) m_glutScreenHeight;
        btScalar w,h;
		const btScalar FOVTG = tanf(m_fovDeg*1.5707963268f/180.0f);	
		if (!m_ortho)	{
			h = m_frustumZFar * FOVTG;			
      	}
        else {
        	h=m_cameraDistance*FOVTG;
			
       	}	
        w=h*aspect;
        const btScalar norX = ((x / (btScalar) m_glutScreenWidth)*2-btScalar(1));
        const btScalar norY = -((y / (btScalar) m_glutScreenHeight)*2-btScalar(1));
        
        const btScalar farX = norX*w;
        const btScalar farY = norY*h;
        
        
        const btVector3 zPlanePoint(-farX,farY,m_frustumZFar);	// in the camera space
        const btVector3 rayTo = m_cameraT.getOrigin() + m_cameraT.getBasis()*zPlanePoint;
        
        return rayTo;
}
void GlutDemoApplicationEx::setShootBoxShape()	{
	const bool recreateMesh = !m_shootBoxShape;
	GlutDemoApplication::setShootBoxShape();
	if (recreateMesh && m_shootBoxShape)	{
		btTriangleMeshEx m_shootBoxMesh;
		const btVector3 he = btUtils::GetAabbHalfExtents(m_shootBoxShape);
		Material mat;	// we will use this material to display the mesh, in case it has no submeshes and no textures and the unique material is the default one (white), because the default white material sucks...
		{
			mat.amb[0] = 0.5f;	mat.amb[1] = 0.5f;	mat.amb[2] = 0.2f;	mat.amb[3] = 1.f;
			mat.dif[0] = 1.f;	mat.dif[1] = 1.f;	mat.dif[2] = 0.f;	mat.dif[3] = 1.f;
			mat.spe[0] = 0.8f;	mat.spe[1] = 0.8f;	mat.spe[2] = 0.6f;	mat.spe[3] = 1.f;
			mat.shi = 0.2f;	// in [0,1], not in the openGL default [0,128]
			mat.emi[0] = 0.f;	mat.emi[1] = 0.f;	mat.emi[2] = 0.f;	mat.emi[3] = 1.f;
		}		
		if (m_shootBoxMesh.loadDefaultBoxMesh(he))	{
			m_shootBoxMesh.setMeshSubPartMaterial(0,mat);
            this->AssignMesh(m_shootBoxShape,m_shootBoxMesh,true);

		}
	}
}
void GlutDemoApplicationEx::shootBox(const btVector3& destination)
{

	if (m_dynamicsWorld)
	{
		
		float mass = 1.f;
		btTransform startTransform;
		startTransform.setIdentity();
                btVector3 camPos = m_cameraT.getOrigin();
                if (m_ortho) startTransform.setOrigin(destination-m_cameraT.getBasis().getColumn(2)*m_frustumZFar);
		else startTransform.setOrigin(camPos);

		setShootBoxShape ();


		btRigidBody* body = this->localCreateRigidBody(mass, startTransform,m_shootBoxShape);
		body->setLinearFactor(btVector3(1,1,1));
		//body->setRestitution(1);

		btVector3 linVel(destination[0]-camPos[0],destination[1]-camPos[1],destination[2]-camPos[2]);
		linVel.normalize();
		linVel*=m_ShootBoxInitialSpeed;

		body->getWorldTransform().setOrigin(startTransform.getOrigin());
		body->getWorldTransform().setRotation(btQuaternion(0,0,0,1));
		body->setLinearVelocity(linVel);
		body->setAngularVelocity(btVector3(0,0,0));
		body->setCcdMotionThreshold(0.5);
		body->setCcdSweptSphereRadius(0.9f);
//		printf("shootBox uid=%d\n", body->getBroadphaseHandle()->getUid());
//		printf("camPos=%f,%f,%f\n",camPos.getX(),camPos.getY(),camPos.getZ());
//		printf("destination=%f,%f,%f\n",destination.getX(),destination.getY(),destination.getZ());
		
	}
}

//--------------------------------------------------------------------------

// Frustum culling stuff----------------------------------------------------
#include "BulletDynamics/Dynamics/btDiscreteDynamicsWorld.h"
#include "BulletDynamics/Dynamics/btRigidBody.h"
#include "LinearMath/btDefaultMotionState.h"
#include "BulletCollision/BroadphaseCollision/btDbvtBroadphase.h"


//#define DEBUG_NUM_OBJECTS_IN_FRUSTUM_WITH_PRINTF

// Here are all the tools needed by "void GlutDemoApplicationEx::performFrustumCulling()":

// Main stuct for btDbvt handling
struct	DbvtBroadphaseFrustumCulling : btDbvt::ICollide	{
                        btAlignedObjectArray<btCollisionObject*>* m_pCollisionObjectArray;
                        short int m_collisionFilterMask;
                        btCollisionObject* m_additionalCollisionObjectToExclude;	// Unused in this demo
                        //-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                        DbvtBroadphaseFrustumCulling(btAlignedObjectArray<btCollisionObject*>* _pArray=NULL)
                                : 	m_pCollisionObjectArray(_pArray),m_collisionFilterMask(btBroadphaseProxy::AllFilter & ~btBroadphaseProxy::SensorTrigger),m_additionalCollisionObjectToExclude(NULL)
                        {
                        }
                        /*
                        bool	Descent(const btDbvtNode* node)	{
                                return false;
                        }
                        */
                        void	Process(const btDbvtNode* node,btScalar depth)	{Process(node);}

                        void	Process(const btDbvtNode* leaf)	{
                                btBroadphaseProxy*	proxy=static_cast < btBroadphaseProxy*> (leaf->data);
                                btCollisionObject* co = static_cast < btCollisionObject* > (proxy->m_clientObject);
                                
                                if ((proxy->m_collisionFilterGroup & m_collisionFilterMask) != 0 && co != m_additionalCollisionObjectToExclude)	{
                                        m_pCollisionObjectArray->push_back( co );
                                }
                        }

}	g_DBFC;

void GlutDemoApplicationEx::reshape(int w, int h) {
    	if (!m_ortho && h>0) calculateAndSetProjectionMatrix(w,h);

        if (w>0 && h>0 && (m_glutScreenWidth != w  || m_glutScreenHeight != h)) {
        	GLDebugResetFont(w,h);
        	m_glutScreenWidth = w;
        	m_glutScreenHeight = h;
        	glViewport(0, 0, w, h);

        	// Occlusion Buffer (we don't scale it, we just correct the aspect ratio)
        	if (w>=h)    {
                const btScalar aspectRatioInverse = (btScalar) h/(btScalar)w;
                
                this->m_raytraceTextureWidth = this->m_raytraceTextureMaxDimension;
                this->m_raytraceTextureHeight = this->m_raytraceTextureWidth * aspectRatioInverse;
            }
            else    {
            	const btScalar aspectRatio = (btScalar) w/(btScalar)h;
                
                this->m_raytraceTextureHeight = this->m_raytraceTextureMaxDimension;
                this->m_raytraceTextureWidth = this->m_raytraceTextureHeight * aspectRatio;                
            }
            // We must delete the previous raytrace buffer:
            /*
            if (m_raytraceTextureId>0)	{
            	glDeleteTextures(1,&m_raytraceTextureId);
            	m_raytraceTextureId=0;
            }
            m_showRaytraceImage = false;
            */
       		if (m_raytraceTextureBuffer) {
				delete[] m_raytraceTextureBuffer;
				m_raytraceTextureBuffer = NULL;
			}
        }
}

// This fills m_objectsInFrustum:
void GlutDemoApplicationEx::performFrustumCulling()	{

        m_objectsInFrustum.resize(0);	// clear() is probably slower
        if (!m_dynamicsWorld) return;

        btDbvtBroadphase*	dbvtBroadphase = dynamic_cast <btDbvtBroadphase*> (m_dynamicsWorld->getBroadphase());
        if (!dbvtBroadphase)	{m_frustumCullingEnabled = false;}
        if (m_frustumCullingEnabled)
        {
                // Perform Dbvt Frustum Culling
                // Mapping stuff:---------------------------------------------------------------------------
                btVector3	dir;
                // We need an updated m_projectionModelviewMatrix = m_projectionMatrix * m_modelviewMatrix;
                Matrix16Helper::Multiply(m_projectionMatrix,m_modelviewMatrix,m_projectionModelviewMatrix);
                // End mapping stuff--------------------------------------------------------------------------

                // Storage stuff------------------
                btVector3 planes_n[5];
                btScalar  planes_o[5];
                //static const int nplanes=sizeof(planes_n)/sizeof(planes_n[0]);
                static const bool cullFarPlane = true;//false;	// This can be tweaked.
                const int	acplanes= cullFarPlane?5:4;
                // End storage stuff---------------

                // Frustum plane extraction--------
                {
                        // This is shorter and cleaner than the approach used in the CDTestFramework
                        // To better understand this approach please browse the web for a pdf named: "Fast Extraction of Viewing Frustum Planes from the World-View-Projection Matrix".

                        const btScalar* mmp = m_projectionModelviewMatrix;
                        // Now we axtract the planes from mmp:

                        // Extract the RIGHT clipping plane
                        planes_n[0]	=	btVector3((btScalar)(mmp[ 3]-mmp[ 0]),(btScalar)(mmp[ 7]-mmp[ 4]),(btScalar)(mmp[11]-mmp[ 8]));
                        // Extract the LEFT clipping plane
                        planes_n[1]	=	btVector3((btScalar)(mmp[ 3]+mmp[ 0]),(btScalar)(mmp[ 7]+mmp[ 4]),(btScalar)(mmp[11]+mmp[ 8]));
                        // Extract the TOP clipping plane
                        planes_n[2]	=	btVector3((btScalar)(mmp[ 3]-mmp[ 1]),(btScalar)(mmp[ 7]-mmp[ 5]),(btScalar)(mmp[11]-mmp[9]));
                        // Extract the BOTTOM clipping plane
                        planes_n[3]	=	btVector3((btScalar)(mmp[ 3]+mmp[ 1]),(btScalar)(mmp[ 7]+mmp[ 5]),(btScalar)(mmp[11]+mmp[9]));
                        // Extract the FAR clipping plane
                        planes_n[4]	=	btVector3((btScalar)(mmp[ 3]-mmp[ 2]),(btScalar)(mmp[ 7]-mmp[ 6]),(btScalar)(mmp[11]-mmp[10]));
                        //planes_n[4]	=	-dir;	//Should work well too... (don't know without normalizations... better stay coherent)
                        dir = -planes_n[4].normalized();

                        // Extract the RIGHT clipping plane
                        planes_o[0] = (btScalar)(mmp[15]-mmp[12]);
                        // Extract the LEFT clipping plane
                        planes_o[1] = (btScalar)(mmp[15]+mmp[12]);
                        // Extract the TOP clipping plane
                        planes_o[2] = (btScalar)(mmp[15]-mmp[13]);
                        // Extract the BOTTOM clipping plane
                        planes_o[3] = (btScalar)(mmp[15]+mmp[13]);
                        // Extract the FAR clipping plane
                        planes_o[4] = (btScalar)(mmp[15]-mmp[14]);
                }
                // End Frustum plane extraction----

                //=======================================================
                // OK, now the pure btDbvt code starts here:
                //=======================================================


                g_DBFC.m_pCollisionObjectArray = &m_objectsInFrustum;
                g_DBFC.m_collisionFilterMask = m_DbvtCullingCollisionFilterMask;//btBroadphaseProxy::AllFilter & ~btBroadphaseProxy::SensorTrigger;	// This won't display sensors...
                g_DBFC.m_additionalCollisionObjectToExclude = NULL;
 

                 btDbvt::collideKDOP(dbvtBroadphase->m_sets[1].m_root,planes_n,planes_o,acplanes,g_DBFC);
                 btDbvt::collideKDOP(dbvtBroadphase->m_sets[0].m_root,planes_n,planes_o,acplanes,g_DBFC);
                 // btDbvt::collideKDOP(root,normals,offsets,count,icollide):
                 // traverse the tree and call ICollide::Process(node) for all leaves located inside/on a set of planes.

                // End Perform Frustum Culling
        }

        #ifdef DEBUG_NUM_OBJECTS_IN_FRUSTUM_WITH_PRINTF
        {
        static int numObjectsInFrustum=0;
        if (m_objectsInFrustum.size()!=numObjectsInFrustum) {
            numObjectsInFrustum = m_objectsInFrustum.size();
            printf("Num objects in frustum = %d\n",m_objectsInFrustum.size());
        }
        }
        #endif //DEBUG_NUM_OBJECTS_IN_FRUSTUM_WITH_PRINTF
}
//---------------------------------------------------------------------------
//-----Raytrace Stuff--------------------------------------------------------
struct btSingleRayCallback : public btBroadphaseRayCallback	{

	btVector3	m_rayFromWorld;
	btVector3	m_rayToWorld;
	btTransform	m_rayFromTrans;
	btTransform	m_rayToTrans;
	btVector3	m_hitNormal;

	const btCollisionWorld*	m_world;
	btCollisionWorld::RayResultCallback&	m_resultCallback;

	btSingleRayCallback(const btVector3& rayFromWorld,const btVector3& rayToWorld,const btCollisionWorld* world,btCollisionWorld::RayResultCallback& resultCallback)
	:m_rayFromWorld(rayFromWorld),
	m_rayToWorld(rayToWorld),
	m_world(world),
	m_resultCallback(resultCallback)
	{
		m_rayFromTrans.setIdentity();
		m_rayFromTrans.setOrigin(m_rayFromWorld);
		m_rayToTrans.setIdentity();
		m_rayToTrans.setOrigin(m_rayToWorld);

		btVector3 rayDir = (rayToWorld-rayFromWorld);

		rayDir.normalize ();
		///what about division by zero? --> just set rayDirection[i] to INF/1e30
		m_rayDirectionInverse[0] = rayDir[0] == btScalar(0.0) ? btScalar(1e30) : btScalar(1.0) / rayDir[0];
		m_rayDirectionInverse[1] = rayDir[1] == btScalar(0.0) ? btScalar(1e30) : btScalar(1.0) / rayDir[1];
		m_rayDirectionInverse[2] = rayDir[2] == btScalar(0.0) ? btScalar(1e30) : btScalar(1.0) / rayDir[2];
		m_signs[0] = m_rayDirectionInverse[0] < 0.0;
		m_signs[1] = m_rayDirectionInverse[1] < 0.0;
		m_signs[2] = m_rayDirectionInverse[2] < 0.0;

		m_lambda_max = rayDir.dot(m_rayToWorld-m_rayFromWorld);

	}
		
	virtual bool	process(const btBroadphaseProxy* proxy)
	{
		///terminate further ray tests, once the closestHitFraction reached zero
		if (m_resultCallback.m_closestHitFraction == btScalar(0.f))
			return false;

		btCollisionObject*	collisionObject = (btCollisionObject*)proxy->m_clientObject;

		//only perform raycast if filterMask matches
		if(m_resultCallback.needsCollision(collisionObject->getBroadphaseHandle())) 
		{
			//RigidcollisionObject* collisionObject = ctrl->GetRigidcollisionObject();
			//btVector3 collisionObjectAabbMin,collisionObjectAabbMax;
#if 0
#ifdef RECALCULATE_AABB
			btVector3 collisionObjectAabbMin,collisionObjectAabbMax;
			collisionObject->getCollisionShape()->getAabb(collisionObject->getWorldTransform(),collisionObjectAabbMin,collisionObjectAabbMax);
#else
			//getBroadphase()->getAabb(collisionObject->getBroadphaseHandle(),collisionObjectAabbMin,collisionObjectAabbMax);
			const btVector3& collisionObjectAabbMin = collisionObject->getBroadphaseHandle()->m_aabbMin;
			const btVector3& collisionObjectAabbMax = collisionObject->getBroadphaseHandle()->m_aabbMax;
#endif
#endif
			//btScalar hitLambda = m_resultCallback.m_closestHitFraction;
			//culling already done by broadphase
			//if (btRayAabb(m_rayFromWorld,m_rayToWorld,collisionObjectAabbMin,collisionObjectAabbMax,hitLambda,m_hitNormal))
			{
				m_world->rayTestSingle(m_rayFromTrans,m_rayToTrans,
					collisionObject,
						collisionObject->getCollisionShape(),
						collisionObject->getWorldTransform(),
						m_resultCallback);
			}
		}
		return true;
	}
};  
void GlutDemoApplicationEx::generateRaytraceImage()	{
        if (!m_dynamicsWorld) return;
	//------Preparation-------------------------------------------------
	{
	 m_showRaytraceImage = false;
	if (m_glutScreenWidth==0 || m_glutScreenHeight==0 || m_raytraceTextureWidth<=0 || m_raytraceTextureHeight<=0) return;
	if (m_raytraceTextureId==0 || !m_raytraceTextureBuffer) {
		if (m_raytraceTextureBuffer)	{
			delete[] m_raytraceTextureBuffer;
			m_raytraceTextureBuffer = NULL;
		}
		if (m_raytraceTextureId>0)	{
			glDeleteTextures(1,&m_raytraceTextureId);
			m_raytraceTextureId=0;
		}
		const long int textureArea = m_raytraceTextureWidth*m_raytraceTextureHeight;
		m_raytraceTextureBuffer = new unsigned char[textureArea];
		if (!m_raytraceTextureBuffer) return;
		for (long int i=0;i<textureArea;i++) m_raytraceTextureBuffer[i]=255;	// Don't know if memset, memcpy, etc. are portable...
		glGenTextures(1,&m_raytraceTextureId);
	}
	}
 	//-------------------------------------------------------------------
 	// Main code
 	//-------------------------------------------------------------------
 	printf("Generating raytrace image... please wait...  \n");
        #define RAYTRACE_OPTION_DARKEN_FAR_REGIONS // This optimization does not affect ortho mode ATM.
        #ifdef RAYTRACE_OPTION_DARKEN_FAR_REGIONS
        btScalar referenceDistance=1;
        if (!m_ortho)
        {
            btVector3 rayTo = getRayToEx(m_glutScreenWidth/2,m_glutScreenHeight/2);
            btVector3 rayFrom = m_cameraT.getOrigin();

            btCollisionWorld::ClosestRayResultCallback rayCallback(rayFrom,rayTo);
            m_dynamicsWorld->rayTest(rayFrom,rayTo,rayCallback);
            if (rayCallback.hasHit())   {
                referenceDistance = rayCallback.m_closestHitFraction;
                if (referenceDistance==0) referenceDistance=SIMD_EPSILON;
                //printf("referenceDistance = %1.3f\n",referenceDistance);
            }
        }
        #endif //RAYTRACE_OPTION_DARKEN_FAR_REGIONS
        {
	unsigned char* pData = &m_raytraceTextureBuffer[0];
	const btScalar w = (btScalar) m_raytraceTextureWidth;
	const btScalar h = (btScalar) m_raytraceTextureHeight;
	
	if (!pData) return;
	//if (m_cameraDistance<m_minCameraDistance)  {m_cameraDistance=m_minCameraDistance;updateCamera();}

	const btScalar screenRatio = (btScalar)m_glutScreenWidth/(btScalar)m_glutScreenHeight;
		
	const btScalar FOVTG= 0.5*tan(m_fovDeg*SIMD_PI/180.0);	

	btTransform T = m_cameraT; 	
	if (m_ortho) T.setOrigin(T.getOrigin()-T.getBasis().getColumn(2)*m_frustumZFar);	// Move back the camera origin (see updateCamera())
	btVector3 rayDir = m_cameraT.getBasis().getColumn(2);
	btScalar rayLength = m_frustumZFar*2;
	rayDir.normalize();
	btVector3 lightNormal = rayDir;
	
	const btScalar NMY = m_ortho ? (m_cameraDistance * FOVTG) : (m_frustumZFar * FOVTG);
	const btScalar NMX = NMY * screenRatio;
	
	// Steps:
	const btScalar DMX = NMX/w;
	const btScalar DMY = NMY/h;
	

	const float ambient = 0.015;
	const unsigned char ambientu = (unsigned char)(ambient*255.0);
	btCollisionWorld::ClosestRayResultCallback cb(T.getOrigin(), T.getOrigin());
	cb.m_flags=0;
	btSingleRayCallback rayCB(T.getOrigin(),T.getOrigin(),m_dynamicsWorld,cb);

	unsigned char C;float Q;	
        btScalar tempX,tempY;
        btVector3 rayToCameraSpace(0,0,0);	// Used if !m_ortho
        btTransform TMX,TMY;		// Used if m_ortho
        TMX.setIdentity();TMY.setIdentity();
	if (!m_ortho)	{
		cb.m_rayFromWorld = rayCB.m_rayFromWorld = m_cameraT.getOrigin();
		rayToCameraSpace.setZ(m_frustumZFar);		
	}		
	for (int y=0;y<h;y++)	{
			tempY = (btScalar(y) - h*0.5);
			if (m_ortho)	{
				TMY = T;
				TMY.getOrigin() = btVector3(TMY.getOrigin() + TMY.getBasis().getColumn(1) * tempY*DMY);
			}
			else rayToCameraSpace.setY(tempY*DMY);	
			
		for (int x=0;x<w;x++)	{
			tempX = (w*0.5 - btScalar(x));
			if (m_ortho)	{
				TMX = TMY;
				TMX.getOrigin() = btVector3(TMX.getOrigin() + TMX.getBasis().getColumn(0) * tempX*DMX);			
			
				cb.m_rayFromWorld = rayCB.m_rayFromWorld = TMX.getOrigin();	
				rayCB.m_rayFromTrans.setIdentity();					
				rayCB.m_rayFromTrans.setOrigin(TMX.getOrigin());

				TMX.getOrigin() = btVector3(TMX.getOrigin() + TMX.getBasis().getColumn(2) * rayLength);
	
				cb.m_rayToWorld = rayCB.m_rayToWorld = TMX.getOrigin();	
			}
			else	{
				tempX = (w*0.5 - btScalar(x));
				rayToCameraSpace.setX(tempX*DMX);
	
				cb.m_rayToWorld = rayCB.m_rayToWorld = m_cameraT.getOrigin()+m_cameraT.getBasis()*rayToCameraSpace;				
			}
			rayCB.m_rayToTrans.setIdentity();				
			rayCB.m_rayToTrans.setOrigin(cb.m_rayToWorld);	
				
			
			 //region Fill 'cb' based on 'cb' and 'rayCB' (Bullet code unwrapped): does not detect soft bodies
			cb.m_closestHitFraction = btScalar(1.);
			cb.m_collisionObject=0;		
			
			rayCB.m_rayDirectionInverse[0] = rayDir[0] == btScalar(0.0) ? btScalar(1e30) : btScalar(1.0) / rayDir[0];
			rayCB.m_rayDirectionInverse[1] = rayDir[1] == btScalar(0.0) ? btScalar(1e30) : btScalar(1.0) / rayDir[1];
			rayCB.m_rayDirectionInverse[2] = rayDir[2] == btScalar(0.0) ? btScalar(1e30) : btScalar(1.0) / rayDir[2];
			rayCB.m_signs[0] = rayCB.m_rayDirectionInverse[0] < 0.0;
			rayCB.m_signs[1] = rayCB.m_rayDirectionInverse[1] < 0.0;
			rayCB.m_signs[2] = rayCB.m_rayDirectionInverse[2] < 0.0;
			rayCB.m_lambda_max = rayDir.dot(rayCB.m_rayToWorld-rayCB.m_rayFromWorld);

			if (!m_frustumCullingEnabled)	{
				/// use the broadphase to accelerate the search for objects, based on their aabb
				/// and for each object with ray-aabb overlap, perform an exact ray test
				m_dynamicsWorld->getBroadphase()->rayTest(rayCB.m_rayFromWorld,rayCB.m_rayToWorld,rayCB);
				// OK, this should be the way to go, but...
				// When we perform frustum culling ourself, I bet it's faster the other branch:
			}	
			else	{
				const btAlignedObjectArray<btCollisionObject*>& collisionObjectArray = m_frustumCullingEnabled ? m_objectsInFrustum : m_dynamicsWorld->getCollisionObjectArray();
				for (int i = 0,sz = collisionObjectArray.size(); i < sz ; i++)	{
					rayCB.process(collisionObjectArray[i]->getBroadphaseHandle());
				}
			}
			
			//endregion
			
			if (cb.hasHit())	{
                                #ifndef RAYTRACE_OPTION_DARKEN_FAR_REGIONS
                                Q = -(cb.m_hitNormalWorld.dot(lightNormal));
                                #else //RAYTRACE_OPTION_DARKEN_FAR_REGIONS
                                if (!m_ortho)   {
                                    const btScalar distFactor = cb.m_closestHitFraction!=0 ? referenceDistance/cb.m_closestHitFraction : 0;  // central ray (focus point).
                                    Q=-(cb.m_hitNormalWorld.dot(lightNormal))*distFactor;
                                }
                                else Q = -(cb.m_hitNormalWorld.dot(lightNormal));
                                #endif //#ifdef RAYTRACE_OPTION_DARKEN_FAR_REGIONS
                                if (Q<0) Q=0;
                                Q+=ambient;
                                if (Q>1) Q=1;
				C = (unsigned char) (Q * 255.0); 
				*pData++ = C;
			}
			else {
				*pData++ = ambientu;			
			}													
		}
	}	
		
	}
 	//-------------------------------------------------------------------
 	// Write m_raytraceTextureBuffer to texture--------------------------
 	{
 	glEnable(GL_TEXTURE_2D);
 	glBindTexture(GL_TEXTURE_2D,m_raytraceTextureId);
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	

	glTexImage2D(GL_TEXTURE_2D, 
		0, 
		GL_LUMINANCE, 
		m_raytraceTextureWidth,m_raytraceTextureHeight, 
		0, 
		GL_LUMINANCE, 
		GL_UNSIGNED_BYTE, 
		&m_raytraceTextureBuffer[0]);
 	m_showRaytraceImage = true;
 	}
	//-----------------------------------------------------------------------
	printf("Raytrace image generated\n");

}
void GlutDemoApplicationEx::drawRaytraceImage(btScalar l,btScalar t,btScalar r,btScalar b)	{
	if (!m_showRaytraceImage || m_raytraceTextureId==0) return;
	glBindTexture(GL_TEXTURE_2D,m_raytraceTextureId);
    glBegin(GL_QUADS);
    glColor4ub(255,255,255,255);
    glNormal3f(0,0,1);
    glTexCoord2f(0,0);glVertex3f(l,b,0);
    glTexCoord2f(1,0);glVertex3f(r,b,0);
    glTexCoord2f(1,1);glVertex3f(r,t,0);
    glTexCoord2f(0,1);glVertex3f(l,t,0);
    glEnd();
}

//-------Alternative Navigation System---------------------------------------	
void GlutDemoApplicationEx::setCameraSpeed(btScalar value) {
	if (value<0) value=0;
	m_alternativeNavigationSystemParams.m_cameraSpeed=value;
}
void GlutDemoApplicationEx::camera_rotate(btScalar x,btScalar y,btScalar z)	{
			const btScalar factor = 0.15f;
			
                m_azi += x*factor;
                m_ele -= y*factor;
                m_roll -= z*factor;
				
				if (m_ele<-90.0) m_ele=-90.0;
				else if (m_ele>90.0) m_ele=90.0;
				if (m_roll<-90.0) m_roll=-90.0;
				else if (m_roll>90.0) m_roll=90.0;

				m_cameraT.setRotation(btQuaternion(btRadians(m_azi),btRadians(m_ele),btRadians(m_roll)));	
				m_cameraUp = m_cameraT.getBasis().getColumn(1);
				//updateCamera();

			
			// Note: we should handle the picking  pivot, but currently it's not possible to
			// rotate when an object has been picked.
			
}
void GlutDemoApplicationEx::camera_move(const btVector3& V)	{
			const btScalar factor = 0.2;
			btVector3 move;
			if (m_ortho && V.z()!=0)	{
				btScalar deltaCameraDistance = btScalar(m_zoomStepSize*btScalar(0.5)*V.z()*m_alternativeNavigationSystemParams.m_cameraSpeed);
				m_cameraDistance -= deltaCameraDistance;
				const btScalar minCameraDistance(0.1);
				if (m_cameraDistance < minCameraDistance) {
					deltaCameraDistance = minCameraDistance - (m_cameraDistance + deltaCameraDistance);
					m_cameraDistance = minCameraDistance;
				}	
				move = - (m_cameraT.getBasis().getColumn(0)*V.x()+m_cameraT.getBasis().getColumn(1)*V.y())*factor*m_alternativeNavigationSystemParams.m_cameraSpeed + m_cameraT.getBasis().getColumn(2)*deltaCameraDistance;
				m_cameraTargetPosition-=m_cameraT.getBasis().getColumn(2)*deltaCameraDistance;
			}
			else	{
			 	move = - (m_cameraT.getBasis().getColumn(0)*V.x()+m_cameraT.getBasis().getColumn(1)*V.y()-m_cameraT.getBasis().getColumn(2)*V.z())*factor*m_alternativeNavigationSystemParams.m_cameraSpeed;
			} 
				
            
			if (m_pickConstraint && !m_ortho)	{
				//move the constraint pivot
				if (m_pickConstraint->getConstraintType() == D6_CONSTRAINT_TYPE)	{
					btGeneric6DofConstraint* pickCon = static_cast<btGeneric6DofConstraint*>(m_pickConstraint);
					if (pickCon)	{
						//keep it at the same picking distance
						btVector3 oldPivotInB = pickCon->getFrameOffsetA().getOrigin();
						btVector3 newPivotB = oldPivotInB + move;;
						pickCon->getFrameOffsetA().setOrigin(newPivotB);
					}
				} 
				else	{
					btPoint2PointConstraint* pickCon = static_cast<btPoint2PointConstraint*>(m_pickConstraint);
					if (pickCon)	{
						//keep it at the same picking distance
						btVector3 oldPivotInB = pickCon->getPivotInB();
						btVector3 newPivotB = oldPivotInB + move;
				
						pickCon->setPivotB(newPivotB);
					}
				}				
			}
				btVector3& v = m_cameraT.getOrigin();
				v+=move;	
				m_cameraPosition = m_cameraT.getOrigin();
				m_cameraUp = m_cameraT.getBasis().getColumn(1);
				//updateCamera();				

			
			//printf("camera: %1.2f,%1.2f,%1.2f\n",m_cameraT.getOrigin().x(),m_cameraT.getOrigin().y(),m_cameraT.getOrigin().z());			
}
//---------------------------------------------------------------------------
//-------Mouse And Keyboard--------------------------------------------------
void GlutDemoApplicationEx::mouseFunc(int button, int state, int x, int y)
{
 
	if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem)	{
		if (state == 0) m_mouseButtons = button;//|= 1<<button;
    	else	{
      		m_mouseButtons = -1;	
		}
    }
    else	 {        
        if (state == 0)	m_mouseButtons |= 1<<button;
     	else m_mouseButtons = 0;
    }

        m_mouseOldX = x;
        m_mouseOldY = y;

        updateModifierKeys();
        
    if (!m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem)	{    
        if ((m_modifierKeys& BT_ACTIVE_ALT) && (state==0)) return;
	}
		
		// button: 0 = LMB, 1 = MMB, 2 = RMB, 
        switch (button)
        {
        case 1:
                {
                		if (state==0)
                        {
								btVector3 rayTo = getRayToEx(x,y);                        
                                shootBox(rayTo);
                        }
                        break;
                };
        /*        
        case 1:
                {


                        if (state==0)
                        {

#if 0
                                //apply an impulse
                                if (m_dynamicsWorld)
                                {
                                		btVector3 rayTo = getRayToEx(x,y);
                        
                                        btCollisionWorld::ClosestRayResultCallback rayCallback(m_cameraPosition,rayTo);
                                        m_dynamicsWorld->rayTest(m_cameraPosition,rayTo,rayCallback);
                                        if (rayCallback.hasHit())
                                        {

                                                btRigidBody* body = btRigidBody::upcast(rayCallback.m_collisionObject);
                                                if (body)
                                                {
                                                        body->setActivationState(ACTIVE_TAG);
                                                        btVector3 impulse = rayTo;
                                                        impulse.normalize();
                                                        float impulseStrength = 10.f;
                                                        impulse *= impulseStrength;
                                                        btVector3 relPos = rayCallback.m_hitPointWorld - body->getCenterOfMassPosition();
                                                        body->applyImpulse(impulse,relPos);
                                                }
                                        }
                                }
#endif



                        } else
                        {

                        }
                        break;
                }
        */        
        case 2:
                {
                        if (state==0)
                        {


                                //add a point to point constraint for picking
                                if (m_dynamicsWorld)
                                {

                                        btVector3 rayTo = getRayToEx(x,y);
                        				
                        				btVector3 rayFrom;
                                        if (m_ortho) rayFrom = rayTo - m_cameraT.getBasis().getColumn(2)*m_frustumZFar;
										else rayFrom = m_cameraPosition;                                  

                                        btCollisionWorld::ClosestRayResultCallback rayCallback(rayFrom,rayTo);
                                        m_dynamicsWorld->rayTest(rayFrom,rayTo,rayCallback);
                                        if (rayCallback.hasHit())
                                        {


                                                btRigidBody* body = const_cast< btRigidBody* >(btRigidBody::upcast(rayCallback.m_collisionObject));
                                                if (body)
                                                {
                                                        //other exclusions?
                                                        if (!(body->isStaticObject() || body->isKinematicObject()))
                                                        {
                                                                pickedBody = body;
                                                                pickedBody->setActivationState(DISABLE_DEACTIVATION);


                                                                btVector3 pickPos = rayCallback.m_hitPointWorld;
                                                                //printf("pickPos=%f,%f,%f\n",pickPos.getX(),pickPos.getY(),pickPos.getZ());


                                                                btVector3 localPivot = body->getCenterOfMassTransform().inverse() * pickPos;






                                                                if (use6Dof)
                                                                {
                                                                        btTransform tr;
                                                                        tr.setIdentity();
                                                                        tr.setOrigin(localPivot);
                                                                        btGeneric6DofConstraint* dof6 = new btGeneric6DofConstraint(*body, tr,false);
                                                                        dof6->setLinearLowerLimit(btVector3(0,0,0));
                                                                        dof6->setLinearUpperLimit(btVector3(0,0,0));
                                                                        dof6->setAngularLowerLimit(btVector3(0,0,0));
                                                                        dof6->setAngularUpperLimit(btVector3(0,0,0));

                                                                        m_dynamicsWorld->addConstraint(dof6);
                                                                        m_pickConstraint = dof6;

                                                                        dof6->setParam(BT_CONSTRAINT_STOP_CFM,0.8,0);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_CFM,0.8,1);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_CFM,0.8,2);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_CFM,0.8,3);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_CFM,0.8,4);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_CFM,0.8,5);

                                                                        dof6->setParam(BT_CONSTRAINT_STOP_ERP,0.1,0);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_ERP,0.1,1);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_ERP,0.1,2);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_ERP,0.1,3);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_ERP,0.1,4);
                                                                        dof6->setParam(BT_CONSTRAINT_STOP_ERP,0.1,5);
                                                                } else
                                                                {
                                                                        btPoint2PointConstraint* p2p = new btPoint2PointConstraint(*body,localPivot);
                                                                        m_dynamicsWorld->addConstraint(p2p);
                                                                        m_pickConstraint = p2p;
                                                                        p2p->m_setting.m_impulseClamp = mousePickClamping;
                                                                        //very weak constraint for picking
                                                                        p2p->m_setting.m_tau = 0.001f;
/*
                                                                        p2p->setParam(BT_CONSTRAINT_CFM,0.8,0);
                                                                        p2p->setParam(BT_CONSTRAINT_CFM,0.8,1);
                                                                        p2p->setParam(BT_CONSTRAINT_CFM,0.8,2);
                                                                        p2p->setParam(BT_CONSTRAINT_ERP,0.1,0);
                                                                        p2p->setParam(BT_CONSTRAINT_ERP,0.1,1);
                                                                        p2p->setParam(BT_CONSTRAINT_ERP,0.1,2);
                                                                        */


                                                                }
                                                                use6Dof = !use6Dof;

                                                                //save mouse position for dragging
                                                                gOldPickingPos = rayTo;
                                                                gHitPos = pickPos;

                                                                gOldPickingDist  = (pickPos-rayFrom).length();
                                                        }
                                                }
                                        }
                                }

                        } else
                        {
                                removePickingConstraint();
                        }

                        break;

                }
        default:
                {
                }
        }

}
void GlutDemoApplicationEx::mouseMotionFunc(int x,int y)
{

	if (m_pickConstraint)
	{
		//move the constraint pivot

		if (m_pickConstraint->getConstraintType() == D6_CONSTRAINT_TYPE)
		{
			btGeneric6DofConstraint* pickCon = static_cast<btGeneric6DofConstraint*>(m_pickConstraint);
			if (pickCon)
			{
				//keep it at the same picking distance

				btVector3 newRayTo = getRayToEx(x,y);
				btVector3 rayFrom;
				btVector3 oldPivotInB = pickCon->getFrameOffsetA().getOrigin();

				btVector3 newPivotB;
				btVector3 dir;
				if (m_ortho) {
					rayFrom = newRayTo - m_cameraT.getBasis().getColumn(2)*m_frustumZFar;
					dir = m_cameraT.getBasis().getColumn(2);
				}	
				else {
					rayFrom = m_cameraPosition;
					dir = newRayTo-rayFrom;
				}	
				dir.normalize();
				dir *= gOldPickingDist;
				newPivotB = rayFrom + dir;
				
				pickCon->getFrameOffsetA().setOrigin(newPivotB);
			}


		} else
		{
			btPoint2PointConstraint* pickCon = static_cast<btPoint2PointConstraint*>(m_pickConstraint);
			if (pickCon)
			{
				//keep it at the same picking distance

				btVector3 newRayTo = getRayToEx(x,y);
				btVector3 rayFrom;
				btVector3 oldPivotInB = pickCon->getPivotInB();
				btVector3 newPivotB;
				btVector3 dir;
				if (m_ortho) {
					rayFrom = newRayTo - m_cameraT.getBasis().getColumn(2)*m_frustumZFar;
					dir = m_cameraT.getBasis().getColumn(2);
				}	
				else {
					rayFrom = m_cameraPosition;
					dir = newRayTo-rayFrom;
				}	
				dir.normalize();
				dir *= gOldPickingDist;
				newPivotB = rayFrom + dir;

				pickCon->setPivotB(newPivotB);
			}
		}
	}

	float dx, dy;
    dx = btScalar(x) - m_mouseOldX;
    dy = btScalar(y) - m_mouseOldY;

	if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem)	{
		if (m_mouseButtons == 0)	{
			camera_rotate(dx,dy);
			//printf("Rotate\n");
		}	
	}
	else	{
	///only if ALT key is pressed (Maya style)
	if (m_modifierKeys& BT_ACTIVE_ALT)
	{
		if(m_mouseButtons & 0)
		{
			btVector3 hor = getRayToEx(0,0)-getRayToEx(1,0);
			btVector3 vert = getRayToEx(0,0)-getRayToEx(0,1);
			btScalar multiplierX = btScalar(0.001);
			btScalar multiplierY = btScalar(0.001);
			if (m_ortho)
			{
				multiplierX = 1;
				multiplierY = 1;
			}


			m_cameraTargetPosition += hor* dx * multiplierX;
			m_cameraTargetPosition += vert* dy * multiplierY;
		}

		if(m_mouseButtons & (2 << 2) && m_mouseButtons & 1)
		{
		}
		else if(m_mouseButtons & 1) 
		{
			m_azi += dx * btScalar(0.2);
			m_azi = fmodf(m_azi, btScalar(360.f));
			m_ele += dy * btScalar(0.2);
			m_ele = fmodf(m_ele, btScalar(180.f));
		} 
		else if(m_mouseButtons & 4) 
		{
			m_cameraDistance -= dy * btScalar(0.02f);
			if (m_cameraDistance<btScalar(0.1))
				m_cameraDistance = btScalar(0.1);

			
		} 
		updateCamera();
	}
	}

	m_mouseOldX = x;
    m_mouseOldY = y;


}
#ifdef __FREEGLUT_EXT_H__
void GlutDemoApplicationEx::mouseWheelFunc(int button,int state/*1 or -1*/,int x,int y)	{
	//printf("mouseWheelFunc(%d,%d,%d,%d)\n",button,state,x,y);
	if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem)	{
		const btScalar linDelta = 4.0*state;
		camera_move(btVector3(0,0,linDelta));
	}
	else if (state>0) zoomIn();
	else zoomOut();	
}
#endif //__FREEGLUT_EXT_H__

void GlutDemoApplicationEx::keyboardCallback(unsigned char key, int x, int y)	{ 
	switch (key)	{
    	case 'a':   
        	setAlternativeNavigationSystemEnabled(!isAlternativeNavigationSystemEnabled());
        	//updateCamera();
      	break;
    	case 'o':   
        	m_ortho = !m_ortho;
        	if (!m_ortho)   reshape(m_glutScreenWidth,m_glutScreenHeight);   // to update projection matrix only
      	break;
    	case '.':
			shootBox(getRayToEx(x,y));
		break;
		case 'R':
			if (m_showRaytraceImage) m_showRaytraceImage = false;
			else generateRaytraceImage();
		break;
		case 'r':
			m_showRaytraceImage = false;
		break;
		default:
    		GlutDemoApplication::keyboardCallback(key,x,y);
    	break;	
    }
}
void GlutDemoApplicationEx::specialKeyboard(int key, int x, int y)	{
	if (!m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem) GlutDemoApplication::specialKeyboard(key,x,y);
	else	{	
	(void)x;
	(void)y;
	
	const btScalar linDelta = 4.0;
	const btScalar angDelta = 2.5;
        //const btScalar factor = 0.2;
	
	switch (key) 
	{
	case GLUT_KEY_F1: if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem) this->setCameraSpeed(this->getCameraSpeed()-0.05);break;
	case GLUT_KEY_F2: if (m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem) this->setCameraSpeed(this->getCameraSpeed()+0.05);break;
	case GLUT_KEY_LEFT : camera_move(btVector3(-linDelta,0,0)); break;
	case GLUT_KEY_RIGHT : camera_move(btVector3(linDelta,0,0)); break;
	case GLUT_KEY_UP : camera_move(btVector3(0,0,linDelta)); break;
	case GLUT_KEY_DOWN : camera_move(btVector3(0,0,-linDelta)); break;	
	case GLUT_KEY_PAGE_UP : camera_move(btVector3(0,-linDelta,0)); break;
	case GLUT_KEY_PAGE_DOWN : camera_move(btVector3(0,linDelta,0)); break;
	case GLUT_KEY_INSERT: camera_rotate(0,0,-angDelta);break;
	case GLUT_KEY_HOME: camera_rotate(0,0,angDelta);break;
	
	default:
		GlutDemoApplication::specialKeyboard(key,x,y);
		//        std::cout << "unused (special) key : " << key << std::endl;
		break;
	}

	//glutPostRedisplay();	 // Needed ?
	}
}
//-------------------------------------------------------------------
//-----Init Exit Physics-----------------------------------------------------
#include <BulletCollision/Gimpact/btGImpactCollisionAlgorithm.h>
#include <BulletCollision/CollisionDispatch/btGhostObject.h>
#include <BulletSoftBody/btSoftRigidDynamicsWorld.h>
#include <BulletSoftBody/btSoftBodyRigidBodyCollisionConfiguration.h>
#include <BulletSoftBody/btSoftBody.h>
void GlutDemoApplicationEx::createEmptyDynamicsWorld(bool softBodySupport)	{
	// Bullet Stuff:	
	///collision configuration contains default setup for memory, collision setup
	m_collisionConfiguration = softBodySupport ? new btSoftBodyRigidBodyCollisionConfiguration() : new btDefaultCollisionConfiguration();
	//m_collisionConfiguration->setConvexConvexMultipointIterations();

	///use the default collision dispatcher. For parallel processing you can use a diffent dispatcher (see Extras/BulletMultiThreaded)
	m_dispatcher = new	btCollisionDispatcher(m_collisionConfiguration);
	if (softBodySupport) {
		if (!m_softBodyWorldInfo) m_softBodyWorldInfo = new btSoftBodyWorldInfo();
		if (m_softBodyWorldInfo) m_softBodyWorldInfo->m_dispatcher = m_dispatcher;
	}	

	m_broadphase = new btDbvtBroadphase();

	///the default constraint solver. For parallel processing you can use a different solver (see Extras/BulletMultiThreaded)
	btSequentialImpulseConstraintSolver* sol = new btSequentialImpulseConstraintSolver;
	m_solver = sol;

	m_dynamicsWorld = softBodySupport ?  new btSoftRigidDynamicsWorld(m_dispatcher,m_broadphase,m_solver,m_collisionConfiguration) : new btDiscreteDynamicsWorld(m_dispatcher,m_broadphase,m_solver,m_collisionConfiguration);
	
	m_dynamicsWorld->setGravity(btVector3(0,-10,0));
	if (m_softBodyWorldInfo) m_softBodyWorldInfo->m_gravity.setValue(0,-10,0);

	// Optional
	//m_dynamicsWorld->getDispatchInfo().m_enableSPU = true;	

	// GIMPACT
	btCollisionDispatcher * dispatcher = static_cast <btCollisionDispatcher *>(m_dynamicsWorld ->getDispatcher());
	btGImpactCollisionAlgorithm::registerAlgorithm(dispatcher);
	
	// NEW => btGhostPairCallback =================================
	m_ghostPairCallback = new btGhostPairCallback(); 
	m_dynamicsWorld->getBroadphase()->getOverlappingPairCache()->setInternalGhostPairCallback(m_ghostPairCallback);	// Needed once to enable ghost objects inside Bullet

	if (m_softBodyWorldInfo) m_softBodyWorldInfo->m_sparsesdf.Initialize();	

}
#include "btCollisionShapeGarbageCollector.h"
void (*g_deleteDynamicsWorld_collisionObjectUserPointerCallback)(void* ptr)=NULL;
// DO NOT USE THE ARRAY BELOW:
btAlignedObjectArray < btCollisionObject* > g_deleteDynamicsWorld_collisionObjectsToExcludeFromDeletion;	// In case you want to add them to another world later (Warning: however displays lists and textures could become dangling pointers)
void GlutDemoApplicationEx::deleteDynamicsWorld(bool deleteTexturesDisplayListsAndOtherStuff)	{
	if (m_dynamicsWorld)	{
	//cleanup in the reverse order of creation/initialization
        //remove the rigidbodies from the dynamics world and delete them
	int i;
	//removed/delete constraints
	for (i=m_dynamicsWorld->getNumConstraints()-1; i>=0 ;i--)
	{
		btTypedConstraint* constraint = m_dynamicsWorld->getConstraint(i);
		m_dynamicsWorld->removeConstraint(constraint);
		delete constraint;
	}	
	for (i=m_dynamicsWorld->getNumCollisionObjects()-1; i>=0 ;i--)
	{
		btCollisionObject* obj = m_dynamicsWorld->getCollisionObjectArray()[i];
		bool saveObjAndItsCollisionShapeButDeleteInnerPointers = false;
		if (g_deleteDynamicsWorld_collisionObjectsToExcludeFromDeletion.size()>0)	{
			if (g_deleteDynamicsWorld_collisionObjectsToExcludeFromDeletion.findLinearSearch(obj)!=g_deleteDynamicsWorld_collisionObjectsToExcludeFromDeletion.size())	{
				m_dynamicsWorld->removeCollisionObject( obj );
				if (!deleteTexturesDisplayListsAndOtherStuff)	continue;
				else saveObjAndItsCollisionShapeButDeleteInnerPointers = true;	// Better to delete all the stuff that can become dangerous later in this case...
																				// Warning: obj->getCollisionShape()->getUserPointer() will still be dangling and will CRASH the program sooner or later...
																				// There's no easy way out here...			
			}
		}
		// If you used SetUserPointer(obj,userPtr)/GetUserPointer(obj) you need to delete it here:-------------------
		if (g_deleteDynamicsWorld_collisionObjectUserPointerCallback)	{
			void* userPtr = GetUserPointer(obj);	// This falls back to obj->getUserPointer() if USE_COLLISION_OBJECT_USER_POINTER is not defined
			if (userPtr) g_deleteDynamicsWorld_collisionObjectUserPointerCallback(userPtr);	
		}
		//---------------------------------------------------------------------------------------------------
		#ifdef USE_COLLISION_OBJECT_USER_POINTER
		btCollisionObjectUserPointer::SafeDeleteUserPointer(obj);	// Just in case it's present. (THIS pointer refers to the user pointer used by the rendering engine to store display lists and overrides on a per object basis)
		#endif //USE_COLLISION_OBJECT_USER_POINTER
		if (saveObjAndItsCollisionShapeButDeleteInnerPointers) continue;
		#ifdef _BT_SOFT_BODY_H
		btSoftBody*	sbody = btSoftBody::upcast(obj);	// Soft bodies delete their own collision shape in their dctr, it can't be shared, and we don't collect it (manually using SetUserPointer() on them is a nonsense, so we don't check it).
		if (!sbody)
		#endif //_BT_SOFT_BODY_H
                if (obj->getCollisionShape()!=m_shootBoxShape)          // m_shootBoxShape gets deleted in this->reset(), so we must avoid to delete it twice.
                            m_collisionShapes.push_back(obj->getCollisionShape());
		btRigidBody* body = btRigidBody::upcast(obj);
		if (body && body->getMotionState())
		{
			delete body->getMotionState();
		}
		m_dynamicsWorld->removeCollisionObject( obj );
		delete obj;
	}

	m_collisionShapes.clear();		// Safely deletes all the collision shapes. 
									// If you used SetUserPointer(collisionShape,userPtr)
									// you DON'T need to set up a m_collisionShapes.setDeleteUserPointerCallback(...) to delete: GetUserPointer(collisionShape).
									// In fact that callback is static and this->GetUserPointer(shape) isn't (due to the member m_shapeDrawer),
									// Instead simply use this->SetDeleteCollisionShapeUserPointerCallback(), and your pointers are deleted
									// when this->reset() is called below. 
	

	delete m_dynamicsWorld;m_dynamicsWorld=NULL;
	}
	if (m_solver) {delete m_solver;m_solver=NULL;}
	
	if (m_broadphase) {delete m_broadphase;m_broadphase=NULL;}
	
	if (m_dispatcher)	{delete m_dispatcher;m_dispatcher=NULL;}

	if (m_collisionConfiguration) {delete m_collisionConfiguration;m_collisionConfiguration=NULL;}
	
	
	if (deleteTexturesDisplayListsAndOtherStuff) {
		this->reset();	// Clears the memory (mainly display lists and textures) associated with the override of GL_ShapeDrawer
                        // Very important: call it AFTER cleaning up everything (even if it deletes user pointers inside collision shapes that do not exist anymore).
                        // Side note: this deletes m_shootBoxShape, but it doesn't set it back to NULL.
    	m_shootBoxShape = NULL; // Mandatory (if we delete it by any means), otherwise it will have a dangling value
    }	

	if (m_ghostPairCallback) {delete m_ghostPairCallback;m_ghostPairCallback=NULL;}
	if (m_softBodyWorldInfo) {delete m_softBodyWorldInfo;m_softBodyWorldInfo=NULL;}

}
//---------------------------------------------------------------------------

