// Based on the original file <DemoApplication.h>
/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it freely,
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/

#ifndef GLUTDEMOAPPLICATIONEX_H__
#define GLUTDEMOAPPLICATIONEX_H__

#define USE_COLLISION_OBJECT_USER_POINTER	// undefine it if you need it yourself, or just use GlutDemoApplicationEx::SetUserPointer(...)/GetUserPointer(...) (needs recompilation)


#include <GlutDemoApplication.h>
#include "btTriangleMeshEx.h"
#include "BulletCollision/CollisionShapes/btCompoundShape.h"
#include "btCollisionShapeGarbageCollector.h"
//#include <GL/freeglut_ext.h>

// Tweakable (This doesn't need recompilation:)
#define USE_BULLET_WORLD_IMPORTER_LIB // IncludeFolder: "Bullet/Extra/Serialize". Needs link to: BulletSerialize.lib


// Works only with Y = up axis and Z = forward axis
struct AlternativeNavigationSystemParams
{
	bool m_enableAlternativeNavigationSystem;
	btVector3 m_cameraStartPosition;
	btScalar m_cameraStartYaw; // in (-SIMD_PI,SIMD_PI)
	btScalar m_cameraStartPitch; // in (-SIMD_HALF_PI,SIMD_HALF_PI)
	btScalar m_cameraStartRoll; // in (-SIMD_HALF_PI,SIMD_HALF_PI)

	btScalar m_cameraSpeed;	//This must be adjusted to match CPU speed (default: 1.0)

	AlternativeNavigationSystemParams() : m_enableAlternativeNavigationSystem(false),m_cameraSpeed(1.0),m_cameraStartPosition(0,25,-100),m_cameraStartYaw(180.0),m_cameraStartPitch(0.0),m_cameraStartRoll(0.0) {}
	AlternativeNavigationSystemParams(bool enableAlternativeNavigationSystem,const btScalar cameraSpeed=btScalar(1.0),const btVector3& cameraStartPosition=btVector3(0,25,-100),const btScalar cameraStartYaw=btScalar(180.0),const btScalar cameraStartPitch=btScalar(0.0),const btScalar cameraStartRoll=btScalar(0.0))
	 : m_enableAlternativeNavigationSystem(enableAlternativeNavigationSystem),m_cameraSpeed(cameraSpeed),m_cameraStartPosition(cameraStartPosition),m_cameraStartYaw(cameraStartYaw),m_cameraStartPitch(cameraStartPitch),m_cameraStartRoll(cameraStartRoll)
	{}

	// Helper Method:
	btTransform getStartCameraTransform() const {
		btQuaternion q(btRadians(m_cameraStartYaw),btRadians(m_cameraStartPitch),btRadians(m_cameraStartRoll));
		btTransform T(q,m_cameraStartPosition);
		return T;
	}
};


class GlutDemoApplicationEx : public GlutDemoApplication	{
	public:

	/// Notes: The physic camera works only with useAlternativeNavigationSystem enabled.
	/// m_ortho works only when useAlternativeNavigationSystem is disabled (because it needs a camera target).
	/// It's up to the user to make sure that the above statements are respected.
	GlutDemoApplicationEx(const AlternativeNavigationSystemParams& alternativeNavigationSystemParams=AlternativeNavigationSystemParams());


	void reset();	//clears memory

	virtual ~GlutDemoApplicationEx();

        void AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals = false,const btVector3& scaling=btVector3(1,1,1),const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray = NULL,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArray = NULL,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray=NULL);
	// Shortcuts mainly used to override materials or textures in meshes with a single submesh:
        void AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride);
        void AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const GLuint firstTextureIdOverride);
        void AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride,const GLuint firstTextureIdOverride);

	void AssignMeshCopy(btCollisionShape* shapeTo,const btCollisionShape* shapeFrom);	// cheaper, because display lists are shared (shapeFrom must have a mesh already assigned to it)
	// ------------------------------------------------------------------------------------------
	const btTriangleMeshExFootPrint::Overrides& GetMeshOverrides(const btCollisionShape& shape) const;
	btTriangleMeshExFootPrint::Overrides& GetMeshOverrides(btCollisionShape& shape);

		// Use this two methods instead of shape->setUserPointer()/shape->getUserPointer() to store user data (that users must manually clean up through a DeleteCollisionShapeUserPointerCallback if necessary)
		void SetUserPointer(btCollisionShape* shape,void* userPointer);
		void* GetUserPointer(const btCollisionShape* shape) const;
		typedef void (*DeleteCollisionShapeUserPointerCallback)(void* collisionShapeUserPointer);
		void SetDeleteCollisionShapeUserPointerCallback(DeleteCollisionShapeUserPointerCallback callback);

        #ifdef USE_COLLISION_OBJECT_USER_POINTER
        void AssignMesh(class btCollisionObject* co,const btTriangleMeshEx& mesh,const bool useFlatNormals = false,const btVector3& scaling=btVector3(1,1,1),const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray = NULL,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArray = NULL,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray=NULL);
	// Shortcuts mainly used to override materials or textures in meshes with a single submesh:
        void AssignMesh(btCollisionObject* co,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride);
        void AssignMesh(btCollisionObject* co,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const GLuint firstTextureIdOverride);
        void AssignMesh(btCollisionObject* co,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const Material& firstMaterialOverride,const GLuint firstTextureIdOverride);

	void AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionShape* shapeFrom);	// cheaper, because display lists are shared (shapeFrom must have a mesh already assigned to it)
	void AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionObject* colObjFrom);	// cheaper, because display lists are shared (colObjFrom must have a mesh already assigned to it)
	// ------------------------------------------------------------------------------------------
	const btTriangleMeshExFootPrint::Overrides& GetMeshOverrides(const btCollisionObject& co) const;
	btTriangleMeshExFootPrint::Overrides& GetMeshOverrides(btCollisionObject& co);
	#endif //USE_COLLISION_OBJECT_USER_POINTER

		// Use this two methods instead of co->setUserPointer()/shape->getUserPointer() to store user data (that users must manually clean up)
		// They fall back to co->setUserPointer()/shape->getUserPointer() if USE_COLLISION_OBJECT_USER_POINTER is not defined
		static void SetUserPointer(class btCollisionObject* co,void* userPointer);
		static void* GetUserPointer(const btCollisionObject* co);


	btRigidBody* localCreateRigidBody(float mass, const btTransform& startTransform,btCollisionShape* shapeWithNoCOMOffset,short int collisionFilterGroup=-1,short int collisionFilterMask=-1);
	btRigidBody* localCreateRigidBodyWithCOMOffset(float mass, const btTransform& startTransform,btCollisionShape* shapeWithNoCOMOffset,const btVector3& COMoffset,short int collisionFilterGroup=-1,short int collisionFilterMask=-1);


	struct btUtils	{

	static bool Save(const btCollisionShape* shape,const char* filename);
	#ifdef USE_BULLET_WORLD_IMPORTER_LIB 	// defined at the top
	inline static btCollisionShape* Load(const char* filename,bool verbose = false);
	#endif //USE_BULLET_WORLD_IMPORTER_LIB


	// Don't even think to use this method when unsupported child shapes (btTriangleShape, btBvhTriangleMeshShape, btGImpact*Shape, btStaticPlaneShape and a bunch more)
	// are present in cs. //TODO: add asserts
	// uniformScalingFactor = 1 can be used when a new compound with the same child shape is needed instead of the same compound shape.
	// In fact a new shape is always returned from this method: this can be useful to map 1 compound shape to exactly 1 rigid body (e.g. glue/fracture,...)
	// Should handle nested compounds (TO BE TESTED)
	// A note on the last argument (deepCloneNestedCompoundsWhenUniformScalingFactorIsOne): when uniformScalingFactor==1 and there are nested compounds (a very unusual case), if
	// this arg is false, they are simply added to the new compound (e.g. useful if the child shapes of the nested compounds can't be glue/fractured themselves on a per rigid body basis),
	// otherwise they are deep cloned using recursion.
	static class btCompoundShape* GetUniformScalingCompoundShape(const btCompoundShape* cs,const btScalar uniformScalingFactor,const bool deepCloneNestedCompoundsWhenUniformScalingFactorIsOne = true);

	// This is instead a very useful method IMO, but I don't know how fast it is.
	static btVector3 GetAabbFullExtents(const btCollisionShape* shape);
    static btVector3 GetAabbHalfExtents(const btCollisionShape* shape);

	protected:
	btUtils() {};

	};

	static int g_colorMode;	// 0 = Bullet Default, 1 = rainbow compounds
	static int g_renderMode;	// 0 = btShapeHull (Bullet Default), 1 = btConvexHullComputer, 2 = Assigned Meshes
	static bool g_cullBackFaces;	//true by default
	static btVector3 g_shadowColor;	// btVector3(0.2,0.2,0.2)

	public:
        friend class GL_MyShapeDrawer;
        virtual void myinit();
        virtual void createEmptyDynamicsWorld(bool softBodySupport = false);
        virtual void deleteDynamicsWorld(bool deleteTexturesDisplayListsAndOtherStuff = true);
        virtual void initPhysics() {};
        virtual void exitPhysics() {};
        virtual void refreshCameraRelatedVariables();
        virtual void updateCamera();
        virtual void calculateAndSetProjectionMatrix(int screenWidth=-1,int screenHeight=-1);
        virtual void calculateAndSetModelviewMatrixFromCameraT();
        virtual void reshape(int w, int h);
        virtual void keyboardCallback(unsigned char key, int x, int y);
        virtual void mouseFunc(int button, int state, int x, int y);
        virtual void mouseMotionFunc(int x,int y);
        virtual void shootBox(const btVector3& destination);
        friend class btCollisionObjectUserPointer;
		//#ifdef __FREEGLUT_EXT_H__
		virtual void mouseWheelFunc(int button,int state/*1 or -1*/,int x,int y);
		//#endif //__FREEGLUT_EXT_H__
		virtual void specialKeyboard(int key, int x, int y);
		//virtual void specialKeyboardUp(int key, int x, int y);

	protected:
		virtual void renderme();
		virtual void rendersceneEx(int pass);	// void DemoApplication::renderscene() is not virtual
        btVector3 getRayToEx(int x,int y);

        // Frustum culling stuff----------------
        bool m_frustumCullingEnabled;
        short int m_DbvtCullingCollisionFilterMask;	// This can be used to further filter objects in frustum (def. btBroadphaseProxy::AllFilter & ~btBroadphaseProxy::SensorTrigger)
        void performFrustumCulling();	// Fills m_objectsInFrustum
        btAlignedObjectArray<btCollisionObject*> m_objectsInFrustum;
        //bool m_sortObjectsInFrustum;	//Experimental
        //--------------------------------------

        // Matrices -----------------------------
        btScalar m_modelviewMatrix[16];
        btScalar m_projectionMatrix[16];
        btScalar m_projectionModelviewMatrix[16];
        btTransform m_cameraT;
        btScalar m_fovDeg;	// It could actually be the HALF of the FOV angle in degrees AFAIK
		//---------------------------------------

		// Raytrace Stuff------------------------
		GLuint m_raytraceTextureId;
		int m_raytraceTextureMaxDimension;
		int m_raytraceTextureWidth;
		int m_raytraceTextureHeight;
		unsigned char* m_raytraceTextureBuffer;
		bool m_showRaytraceImage;
		virtual void generateRaytraceImage();
		void drawRaytraceImage(btScalar l,btScalar t,btScalar r,btScalar b);
		//-----------------------------------------
		// Alternative navigation system
		private:
		AlternativeNavigationSystemParams m_alternativeNavigationSystemParams;	// This can be changed at runtime using setAlternativeNavigationSystemEnabled
		protected:
		// works only with +Y = up axis and +> = forward axis
		void setAlternativeNavigationSystemEnabled(bool flag, bool forceConversion=false);	// Please use this for runtime switch only. Otherwise just use params in the ctr
		bool isAlternativeNavigationSystemEnabled() const {return m_alternativeNavigationSystemParams.m_enableAlternativeNavigationSystem;}
		/// The camera speed value is used only when "useAlternativeNavigationSystem" is set to "true" in the ctr.
		void setCameraSpeed(btScalar value);
		/// The camera speed value is used only when "useAlternativeNavigationSystem" is set to "true" in the ctr.
		inline btScalar getCameraSpeed() const {return m_alternativeNavigationSystemParams.m_cameraSpeed;}
		virtual void camera_rotate(btScalar x,btScalar y,btScalar z=btScalar(0));
		virtual void camera_move(const btVector3& V);
		float m_roll;
		//------------------------------------------
		class btBroadphaseInterface*	m_broadphase;
		class btCollisionDispatcher*	m_dispatcher;
		class btConstraintSolver*	m_solver;
		class btDefaultCollisionConfiguration* m_collisionConfiguration;
		class btSoftBodyWorldInfo* m_softBodyWorldInfo;
                btCollisionShapeGarbageCollector m_collisionShapes;
		static GlutDemoApplicationEx* g_myGlutDemoApplicationEx;	// may turn useful for redirecting global callbacks to members of this class...

		private:
		bool m_areDefaultTextureSettingsEnabled;
		class btGhostPairCallback* m_ghostPairCallback;				// Needed once to enable ghost objects inside Bullet

		protected:

		virtual void setShootBoxShape();
};

#ifdef USE_COLLISION_OBJECT_USER_POINTER
class GlutDemoApplicationEx;
ATTRIBUTE_ALIGNED16(class)	btCollisionObjectUserPointer
{
protected:
	btTriangleMeshExFootPrint* customMesh;	// if set it's deleted by dctr (OWNED)
	btTriangleMeshExFootPrint::Overrides overrides;
	void* userPointer;

	static btCollisionObjectUserPointer* GetFrom(const btCollisionObject* obj);	// Creates an empty one if needed (with a const_cast)
	friend void GlutDemoApplicationEx::rendersceneEx(int pass);
    friend void GlutDemoApplicationEx::AssignMesh(btCollisionObject* co,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArrayGlutDemoApplicationEx,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray);
	friend void GlutDemoApplicationEx::AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionShape* shapeFrom);
	friend void GlutDemoApplicationEx::AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionObject* colObjFrom);
	friend const btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(const btCollisionObject& co) const;
	friend btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(btCollisionObject& co);
	friend void GlutDemoApplicationEx::SetUserPointer(btCollisionObject* co,void* userPointer);
	friend void* GlutDemoApplicationEx::GetUserPointer(const btCollisionObject* co);
        friend struct btCollisionObjectCompareFunc;
public:
	BT_DECLARE_ALIGNED_ALLOCATOR();

	btCollisionObjectUserPointer();
	virtual ~btCollisionObjectUserPointer();

	static void SafeDeleteUserPointer(btCollisionObject* obj);
};
#include "LinearMath/btDefaultMotionState.h"
class	btMyDefaultMotionState : public btDefaultMotionState	{
	public:
	btVector3 centerOfMassOffset;
	// Note that here we don't use: btTransform m_centerOfMassOffset (in btDefaultMotionState) at all.
	btMyDefaultMotionState(const btTransform& startTrans = btTransform::getIdentity(),const btVector3& _centerOfMassOffset=btVector3(0,0,0));
	static btMyDefaultMotionState* upcast(btRigidBody* co);
	static const btMyDefaultMotionState* upcast(const btRigidBody* co);

	virtual void getWorldTransformOutsideCOM(btTransform& worldTransOutsideCOM) const;
	virtual void setWorldTransformOutsideCOM(const btTransform& worldTransOutsideCOM);
	// We leave the outher 2 get/set methods available for the position of the COM:
	///synchronizes world transform from user to physics
	virtual void getWorldTransform(btTransform& centerOfMassWorldTrans ) const;
	///synchronizes world transform from physics to user
	///Bullet only calls the update of worldtransform for active objects
	virtual void setWorldTransform(const btTransform& centerOfMassWorldTrans);

	virtual ~btMyDefaultMotionState();
};
#endif //USE_COLLISION_OBJECT_USER_POINTER

class GL_MyShapeDrawer : public GL_ShapeDrawer	{
	protected:
        //ATTRIBUTE_ALIGNED16(struct) MyShapeCache
        struct MyShapeCache
		{
		ShapeCache shapeCache;		// Note that I cannot derive MyShapeCache from ShapeCache, since ShapeCache is not a polymorphic struct!
		MyShapeCache(btConvexShape* s);
		GLuint dl[2];
		btTriangleMeshExFootPrint* customMesh;	// if set it's deleted by dctr (OWNED)
		~MyShapeCache();
		btTriangleMeshExFootPrint::Overrides overrides;

		// These 2 ptrs are set by btRigidBody->getUserPointer() and reset after each drawing
		const btTriangleMeshExFootPrint* customMeshGuest;				// mutable data (don't use)
		const btTriangleMeshExFootPrint::Overrides* overridesGuest;		// mutable data (don't use)

		void* userPointer;			// this can be used by the user, as long as he deletes it at the end.
                //BT_DECLARE_ALIGNED_ALLOCATOR();
        };
	MyShapeCache*							mycache(btCollisionShape*);
	typedef void (*DeleteCollisionShapeUserPointerCallback)(void* collisionShapeUserPointer);
	DeleteCollisionShapeUserPointerCallback deleteCollisionShapeUserPointerCallback;

	//clean-up memory of dynamically created shape hulls
	btAlignedObjectArray<MyShapeCache*>	m_myshapecaches;
    friend void GlutDemoApplicationEx::AssignMesh(btCollisionShape* shape,const btTriangleMeshEx& mesh,const bool useFlatNormals,const btVector3& scaling,const btAlignedObjectArray< const Material* >* pOptionalMaterialOverrideArray,const btAlignedObjectArray< GLuint >* pOptionalTextureOverrideArrayGlutDemoApplicationEx,const btAlignedObjectArray< int >* pOptionalSubmeshIndexArray);
	friend void GlutDemoApplicationEx::AssignMeshCopy(btCollisionShape* shapeTo,const btCollisionShape* shapeFrom);
    #ifdef USE_COLLISION_OBJECT_USER_POINTER
	friend void GlutDemoApplicationEx::AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionShape* shapeFrom);
	friend void GlutDemoApplicationEx::AssignMeshCopy(btCollisionObject* colObjTo,const btCollisionObject* colObjFrom);
	#endif //USE_COLLISION_OBJECT_USER_POINTER
	friend void GlutDemoApplicationEx::renderme();
	friend void GlutDemoApplicationEx::rendersceneEx(int pass);
	friend const btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(const btCollisionShape& shape) const;
	friend btTriangleMeshExFootPrint::Overrides& GlutDemoApplicationEx::GetMeshOverrides(btCollisionShape& shape);
	friend struct DbvtBroadphaseFrustumCulling;
	friend void GlutDemoApplicationEx::SetUserPointer(btCollisionShape* shape,void* userPointer);
	friend void* GlutDemoApplicationEx::GetUserPointer(const btCollisionShape* shape) const;
	friend void GlutDemoApplicationEx::SetDeleteCollisionShapeUserPointerCallback(GlutDemoApplicationEx::DeleteCollisionShapeUserPointerCallback callback);
	void setDeleteCollisionShapeUserPointerCallback(DeleteCollisionShapeUserPointerCallback callback) {deleteCollisionShapeUserPointerCallback = callback;}
	friend void GlutDemoApplicationEx::reset();
        friend struct btCollisionObjectCompareFunc;
        private:
	bool* m_areDefaultTextureSettingsEnabled;

	public:

	virtual void		drawShadow(btScalar* m, const btVector3& extrusion,const btCollisionShape* shape,const btVector3& worldBoundsMin,const btVector3& worldBoundsMax);
	GL_MyShapeDrawer() : deleteCollisionShapeUserPointerCallback(NULL)	{}
	virtual ~GL_MyShapeDrawer();
	virtual void		drawOpenGL(btScalar* m, const btCollisionShape* shape, const btVector3& color,int	debugMode,const btVector3& worldBoundsMin,const btVector3& worldBoundsMax,const int pass = 0);
};

#ifdef USE_BULLET_WORLD_IMPORTER_LIB
#include <BulletWorldImporter/btBulletWorldImporter.h>
btCollisionShape* GlutDemoApplicationEx::btUtils::Load(const char* filename,bool verbose)	{

	btBulletWorldImporter loader(0);//don't store info into the world
	loader.setVerboseMode(verbose);
	if (!loader.loadFile(filename)) return NULL;

	//int	loader.getNumCollisionShapes() const;
	//btCollisionShape* loader.getCollisionShapeByIndex(int index);
	//btCollisionShape* loader.getCollisionShapeByName(const char* name);

	btCollisionShape* shape = NULL;
	if (loader.getNumCollisionShapes()>0) shape = loader.getCollisionShapeByIndex(0);

	//btCollisionShape* shape = loader.getCollisionShapeByName("BCS");

	//TODO: Cleaner way:
	// 1) Deep clone Collision Shape
	// 2) loader.deleteAllData(); // deletes all (collision shapes included)
	// 3) return Deep cloned Collision Shape

	// Here we don't delete all data from the loader. (leaks?)
	return shape;
}
#endif //USE_BULLET_WORLD_IMPORTER_LIB

// Frustum culling stuff----------------
#include "BulletCollision/CollisionDispatch/btCollisionObject.h"
enum NewCollisionFlags	//object->getCollisionFlags()
        {
                /*
                CF_STATIC_OBJECT= 1,
                CF_KINEMATIC_OBJECT= 2,
                CF_NO_CONTACT_RESPONSE = 4,			//object->hasContactResponse()
                CF_CUSTOM_MATERIAL_CALLBACK = 8,//this allows per-triangle material (friction/restitution)
                CF_CHARACTER_OBJECT = 16,
                CF_DISABLE_VISUALIZE_OBJECT = 32, //disable debug drawing
                CF_DISABLE_SPU_COLLISION_PROCESSING = 64,//disable parallel/SPU processing
                */
                // New:
                CF_NO_SHADOW_CASTER = 2048,
                CF_NO_SHADOW_RECEIVER = 4096
        };
//--------------------------------------

class btMatrix3x3Helper	{
	// The more I use these methods, the more I wonder why they are not present inside the btMatrix3x3 class...
	// However I believe that they're a bit slow... and could be better optimized.
	public:

	// Code adapted from the Ogre Source Code (OgreMatrix3.cpp), (www.ogre3d.org, MIT License)
	SIMD_FORCE_INLINE static void FromEulerYXZ(btMatrix3x3& m,btScalar fYAngle,btScalar fPAngle) {
      	//#define TEST_FASTER_STUFF
      	#ifndef TEST_FASTER_STUFF
      	btScalar fCos, fSin;

        fCos = btCos(fYAngle);
        fSin = btSin(fYAngle);
        btMatrix3x3 kYMat(fCos,0.0,fSin,0.0,1.0,0.0,-fSin,0.0,fCos);

        fCos = btCos(fPAngle);
        fSin = btSin(fPAngle);
        btMatrix3x3 kXMat(1.0,0.0,0.0,0.0,fCos,-fSin,0.0,fSin,fCos);

        fCos = btScalar(1.f);//btCos(fRAngle);
        fSin = btScalar(0.f);//btSin(fRAngle);
        btMatrix3x3 kZMat(fCos,-fSin,0.0,fSin,fCos,0.0,0.0,0.0,1.0);

        m = kYMat*(kXMat*kZMat);
        #else //TEST_FASTER_STUFF
        // Isn't this faster? To test
        //m = btMatrix3x3(btQuaternion(btVector3(0,1,0),fYAngle)) * btMatrix3x3(btQuaternion(btVector3(1,0,0),fPAngle));
		m.setRotation(btQuaternion(btVector3(0,1,0),fYAngle)*btQuaternion(btVector3(1,0,0),fPAngle));
		#undef TEST_FASTER_STUFF
		#endif //TEST_FASTER_STUFF
	}
	// Code adapted from the Ogre Source Code (OgreMatrix3.cpp), (www.ogre3d.org, MIT License)
	SIMD_FORCE_INLINE static bool ToEulerYXZ(const btMatrix3x3& m,btScalar& rfYAngle,btScalar& rfPAngle) {
        // rot =  cy*cz+sx*sy*sz  cz*sx*sy-cy*sz  cx*sy
        //        cx*sz           cx*cz          -sx
        //       -cz*sy+cy*sx*sz  cy*cz*sx+sy*sz  cx*cy

        rfPAngle = btAsin(-m[1][2]);
        if ( rfPAngle < SIMD_HALF_PI )
        {
            if ( rfPAngle > -SIMD_HALF_PI )
            {
                rfYAngle = btAtan2(m[0][2],m[2][2]);
                return true;
            }
            else
            {
                // WARNING.  Not a unique solution.
                rfYAngle = -btAtan2(-m[0][1],m[0][0]);
                return false;
            }
        }
        else
        {
            // WARNING.  Not a unique solution.
            rfYAngle = btAtan2(-m[0][1],m[0][0]);
            return false;
        }
	}

	protected:
	btMatrix3x3Helper() {}
};


#endif



